@@ -1,6 +1,6 @@
 /*
- * hurl (https://hurl.dev)
- * Copyright (C) 2020 Orange
+ * Hurl (https://hurl.dev)
+ * Copyright (C) 2022 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,18 +16,22 @@
  *
  */
 
-use crate::cli;
-use crate::cli::CliError;
-use crate::http::ClientOptions;
-use crate::runner::Value;
-use atty::Stream;
-use clap::{App, AppSettings, ArgMatches};
 use std::collections::HashMap;
+use std::env;
 use std::fs::File;
 use std::io::{BufRead, BufReader};
 use std::path::{Path, PathBuf};
 use std::time::Duration;
 
+use atty::Stream;
+use clap::{AppSettings, ArgAction, ArgMatches, Command};
+
+use crate::cli;
+use crate::cli::CliError;
+use crate::http::ClientOptions;
+use crate::runner::Value;
+use crate::util::logger::BaseLogger;
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct CliOptions {
     pub cacert_file: Option<String>,
@@ -45,7 +49,7 @@ pub struct CliOptions {
     pub include: bool,
     pub insecure: bool,
     pub interactive: bool,
-    pub junit_file: Option<PathBuf>,
+    pub junit_file: Option<String>,
     pub max_redirect: Option<usize>,
     pub no_proxy: Option<String>,
     pub output: Option<String>,
@@ -56,8 +60,10 @@ pub struct CliOptions {
     pub timeout: Duration,
     pub to_entry: Option<usize>,
     pub user: Option<String>,
+    pub user_agent: Option<String>,
     pub variables: HashMap<String, Value>,
     pub verbose: bool,
+    pub very_verbose: bool,
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -66,196 +72,190 @@ pub enum OutputType {
     Json,
     NoOutput,
 }
-pub fn app() -> App<'static, 'static> {
-    App::new("hurl")
+
+pub fn app(version: &str) -> Command {
+    Command::new("hurl")
         .about("Run hurl FILE(s) or standard input")
         .setting(AppSettings::DeriveDisplayOrder)
-        .setting(AppSettings::UnifiedHelpMessage)
+        .disable_colored_help(true)
+        .version(version)
         .arg(
-            clap::Arg::with_name("INPUT")
+            clap::Arg::new("INPUT")
                 .help("Sets the input file to use")
                 .required(false)
-                .multiple(true),
+                .multiple_values(true),
         )
         .arg(
-            clap::Arg::with_name("cacert_file")
+            clap::Arg::new("cacert_file")
                 .long("cacert")
                 .value_name("FILE")
                 .help("CA certificate to verify peer against (PEM format)"),
         )
         .arg(
-            clap::Arg::with_name("color")
+            clap::Arg::new("color")
                 .long("color")
                 .conflicts_with("no_color")
                 .help("Colorize Output"),
         )
         .arg(
-            clap::Arg::with_name("compressed")
+            clap::Arg::new("compressed")
                 .long("compressed")
                 .help("Request compressed response (using deflate or gzip)"),
         )
         .arg(
-            clap::Arg::with_name("connect_timeout")
+            clap::Arg::new("connect_timeout")
                 .long("connect-timeout")
                 .value_name("SECONDS")
                 .help("Maximum time allowed for connection"),
         )
         .arg(
-            clap::Arg::with_name("cookies_input_file")
-                .short("b")
+            clap::Arg::new("cookies_input_file")
+                .short('b')
                 .long("cookie")
                 .value_name("FILE")
                 .help("Read cookies from FILE"),
         )
         .arg(
-            clap::Arg::with_name("cookies_output_file")
-                .short("c")
+            clap::Arg::new("cookies_output_file")
+                .short('c')
                 .long("cookie-jar")
                 .value_name("FILE")
                 .help("Write cookies to FILE after running the session (only for one session)"),
         )
         .arg(
-            clap::Arg::with_name("fail_at_end")
+            clap::Arg::new("fail_at_end")
                 .long("fail-at-end")
                 .help("Fail at end")
-                .takes_value(false),
         )
         .arg(
-            clap::Arg::with_name("file_root")
+            clap::Arg::new("file_root")
                 .long("file-root")
                 .value_name("DIR")
                 .help("set root filesystem to import file in hurl (default is current directory)")
                 .takes_value(true),
         )
         .arg(
-            clap::Arg::with_name("follow_location")
-                .short("L")
+            clap::Arg::new("follow_location")
+                .short('L')
                 .long("location")
                 .help("Follow redirects"),
         )
         .arg(
-            clap::Arg::with_name("glob")
+            clap::Arg::new("glob")
                 .long("glob")
                 .value_name("GLOB")
-                .multiple(true)
+                .action(ArgAction::Append)
                 .number_of_values(1)
                 .help("Specify input files that match the given blob. Multiple glob flags may be used."),
         )
         .arg(
-            clap::Arg::with_name("html")
-                .long("html")
-                .value_name("DIR")
-                .help("Generate html report to dir")
-                .takes_value(true),
-        )
-        .arg(
-            clap::Arg::with_name("include")
-                .short("i")
+            clap::Arg::new("include")
+                .short('i')
                 .long("include")
                 .help("Include the HTTP headers in the output"),
         )
         .arg(
-            clap::Arg::with_name("ignore_asserts")
+            clap::Arg::new("ignore_asserts")
                 .long("ignore-asserts")
                 .help("Ignore asserts defined in the Hurl file."),
         )
         .arg(
-            clap::Arg::with_name("insecure")
-                .short("k")
+            clap::Arg::new("insecure")
+                .short('k')
                 .long("insecure")
                 .help("Allow insecure SSL connections"),
         )
         .arg(
-            clap::Arg::with_name("interactive")
+            clap::Arg::new("interactive")
                 .long("interactive")
                 .conflicts_with("to_entry")
                 .help("Turn on interactive mode"),
         )
         .arg(
-            clap::Arg::with_name("json")
+            clap::Arg::new("json")
                 .long("json")
                 .conflicts_with("no_output")
                 .help("Output each hurl file result to JSON"),
         )
         .arg(
-            clap::Arg::with_name("junit")
-                .long("report-junit")
-                .value_name("FILE")
-                .help("Write a Junit XML report to the given file")
-                .takes_value(true),
-        )
-        .arg(
-            clap::Arg::with_name("max_time")
-                .long("max-time")
-                .short("m")
-                .value_name("NUM")
-                .allow_hyphen_values(true)
-                .help("Maximum time allowed for the transfer"),
-        )
-        .arg(
-            clap::Arg::with_name("max_redirects")
+            clap::Arg::new("max_redirects")
                 .long("max-redirs")
                 .value_name("NUM")
                 .allow_hyphen_values(true)
                 .help("Maximum number of redirects allowed"),
         )
         .arg(
-            clap::Arg::with_name("no_color")
+            clap::Arg::new("max_time")
+                .long("max-time")
+                .short('m')
+                .value_name("NUM")
+                .allow_hyphen_values(true)
+                .help("Maximum time allowed for the transfer"),
+        )
+        .arg(
+            clap::Arg::new("no_color")
                 .long("no-color")
                 .conflicts_with("color")
                 .help("Do not colorize Output"),
         )
         .arg(
-            clap::Arg::with_name("no_output")
+            clap::Arg::new("no_output")
                 .long("no-output")
                 .conflicts_with("json")
                 .help("Suppress output. By default, Hurl outputs the body of the last response."),
         )
         .arg(
-            clap::Arg::with_name("noproxy")
+            clap::Arg::new("noproxy")
                 .long("noproxy")
                 .value_name("HOST(S)")
                 .help("List of hosts which do not use proxy")
                 .takes_value(true),
         )
         .arg(
-            clap::Arg::with_name("output")
-                .short("o")
+            clap::Arg::new("output")
+                .short('o')
                 .long("output")
                 .value_name("FILE")
                 .help("Write to FILE instead of stdout"),
         )
         .arg(
-            clap::Arg::with_name("progress")
+            clap::Arg::new("progress")
                 .long("progress")
-                .help("Print filename and status for each test (stderr)"),
+                .help("Print filename and status for each test (stderr) - deprecated use --test"),
         )
         .arg(
-            clap::Arg::with_name("proxy")
-                .short("x")
+            clap::Arg::new("proxy")
+                .short('x')
                 .long("proxy")
                 .value_name("[PROTOCOL://]HOST[:PORT]")
                 .help("Use proxy on given protocol/host/port"),
         )
         .arg(
-            clap::Arg::with_name("report_html")
+            clap::Arg::new("junit")
+                .long("report-junit")
+                .value_name("FILE")
+                .help("Write a Junit XML report to the given file")
+                .takes_value(true),
+        )
+        .arg(
+            clap::Arg::new("report_html")
                 .long("report-html")
                 .value_name("DIR")
                 .help("Generate html report to dir")
                 .takes_value(true),
         )
         .arg(
-            clap::Arg::with_name("summary")
+            clap::Arg::new("summary")
                 .long("summary")
-                .help("Print test metrics at the end of the run (stderr)"),
+                .help("Print test metrics at the end of the run (stderr) - deprecated use --test"),
         )
         .arg(
-            clap::Arg::with_name("test")
+            clap::Arg::new("test")
                 .long("test")
-                .help("Activate test mode; equals --no-output --progress --summary"),
+                .help("Activate test mode."),
         )
         .arg(
-            clap::Arg::with_name("to_entry")
+            clap::Arg::new("to_entry")
                 .long("to-entry")
                 .value_name("ENTRY_NUMBER")
                 .conflicts_with("interactive")
@@ -263,52 +263,65 @@ pub fn app() -> App<'static, 'static> {
                 .takes_value(true),
         )
         .arg(
-            clap::Arg::with_name("user")
-                .short("u")
+            clap::Arg::new("user")
+                .short('u')
                 .long("user")
                 .value_name("user:password")
                 .help("Add basic Authentication header to each request.")
                 .takes_value(true),
         )
         .arg(
-            clap::Arg::with_name("variable")
+            clap::Arg::new("user_agent")
+                .short('A')
+                .long("user-agent")
+                .value_name("name")
+                .help("Specify the User-Agent string to send to the HTTP server.")
+                .takes_value(true),
+        )
+        .arg(
+            clap::Arg::new("variable")
                 .long("variable")
                 .value_name("NAME=VALUE")
-                .multiple(true)
+                .action(ArgAction::Append)
                 .number_of_values(1)
                 .help("Define a variable")
                 .takes_value(true),
         )
         .arg(
-            clap::Arg::with_name("variables_file")
+            clap::Arg::new("variables_file")
                 .long("variables-file")
                 .value_name("FILE")
                 .help("Define a properties file in which you define your variables")
                 .takes_value(true),
         )
         .arg(
-            clap::Arg::with_name("verbose")
-                .short("v")
+            clap::Arg::new("verbose")
+                .short('v')
                 .long("verbose")
                 .help("Turn on verbose output"),
         )
+        .arg(
+            clap::Arg::new("very_verbose")
+                .long("very-verbose")
+                .help("Turn on verbose output, uncluding HTTP response"),
+        )
 }
 
-pub fn parse_options(matches: ArgMatches) -> Result<CliOptions, CliError> {
-    let cacert_file = match matches.value_of("cacert_file") {
+pub fn parse_options(matches: &ArgMatches, logger: &BaseLogger) -> Result<CliOptions, CliError> {
+    let cacert_file = match get_string(matches, "cacert_file") {
         None => None,
         Some(filename) => {
-            if !Path::new(filename).is_file() {
+            if !Path::new(&filename).is_file() {
                 let message = format!("File {} does not exist", filename);
                 return Err(CliError { message });
             } else {
-                Some(filename.to_string())
+                Some(filename)
             }
         }
     };
-    let color = output_color(matches.clone());
-    let compressed = matches.is_present("compressed");
-    let connect_timeout = match matches.value_of("connect_timeout") {
+    let color = output_color(matches);
+    let compressed = has_flag(matches, "compressed");
+    let connect_timeout = match get_string(matches, "connect_timeout") {
         None => ClientOptions::default().connect_timeout,
         Some(s) => match s.parse::<u64>() {
             Ok(n) => Duration::from_secs(n),
@@ -319,29 +332,15 @@ pub fn parse_options(matches: ArgMatches) -> Result<CliOptions, CliError> {
             }
         },
     };
-    let cookie_input_file = matches
-        .value_of("cookies_input_file")
-        .map(|x| x.to_string());
-    let cookie_output_file = matches
-        .value_of("cookies_output_file")
-        .map(|x| x.to_string());
-    let fail_fast = !matches.is_present("fail_at_end");
-    let file_root = matches.value_of("file_root").map(|value| value.to_string());
-    let follow_location = matches.is_present("follow_location");
-    let glob_files = match_glob_files(matches.clone())?;
-    // deprecated
-    // Support --report-html and --html only for the current version
-    if matches.is_present("html") {
-        eprintln!("The option --html is deprecated. It has been renamed to --report-html.");
-        eprintln!("It will be removed in the next version");
-    }
-    let report_html = if let Some(dir) = matches.value_of("html") {
-        Some(dir)
-    } else {
-        matches.value_of("report_html")
-    };
+    let cookie_input_file = get_string(matches, "cookies_input_file");
+    let cookie_output_file = get_string(matches, "cookies_output_file");
+    let fail_fast = !has_flag(matches, "fail_at_end");
+    let file_root = get_string(matches, "file_root");
+    let follow_location = has_flag(matches, "follow_location");
+    let glob_files = match_glob_files(matches)?;
+    let report_html = get_string(matches, "report_html");
     let html_dir = if let Some(dir) = report_html {
-        let path = Path::new(dir);
+        let path = Path::new(&dir);
         if !path.exists() {
             match std::fs::create_dir(path) {
                 Err(_) => {
@@ -361,17 +360,12 @@ pub fn parse_options(matches: ArgMatches) -> Result<CliOptions, CliError> {
     } else {
         None
     };
-    let ignore_asserts = matches.is_present("ignore_asserts");
-    let include = matches.is_present("include");
-    let insecure = matches.is_present("insecure");
-    let interactive = matches.is_present("interactive");
-    let junit_file = if let Some(filename) = matches.value_of("junit") {
-        let path = Path::new(filename);
-        Some(path.to_path_buf())
-    } else {
-        None
-    };
-    let max_redirect = match matches.value_of("max_redirects") {
+    let ignore_asserts = has_flag(matches, "ignore_asserts");
+    let include = has_flag(matches, "include");
+    let insecure = has_flag(matches, "insecure");
+    let interactive = has_flag(matches, "interactive");
+    let junit_file = get_string(matches, "junit");
+    let max_redirect = match get_string(matches, "max_redirects").as_deref() {
         None => Some(50),
         Some("-1") => None,
         Some(s) => match s.parse::<usize>() {
@@ -383,21 +377,32 @@ pub fn parse_options(matches: ArgMatches) -> Result<CliOptions, CliError> {
             }
         },
     };
-    let no_proxy = matches.value_of("proxy").map(|x| x.to_string());
-    let output = matches
-        .value_of("output")
-        .map(|filename| filename.to_string());
-    let output_type = if matches.is_present("json") {
+    let no_proxy = get_string(matches, "proxy");
+    let output = get_string(matches, "output");
+    let test = has_flag(matches, "test");
+    let output_type = if has_flag(matches, "json") {
         OutputType::Json
-    } else if matches.is_present("no_output") || matches.is_present("test") {
+    } else if has_flag(matches, "no_output") || test {
         OutputType::NoOutput
     } else {
         OutputType::ResponseBody
     };
-    let progress = matches.is_present("progress") || matches.is_present("test");
-    let proxy = matches.value_of("proxy").map(|x| x.to_string());
-    let summary = matches.is_present("summary") || matches.is_present("test");
-    let timeout = match matches.value_of("max_time") {
+    let progress = has_flag(matches, "progress");
+    if progress {
+        logger.warning(
+            "--progress option is deprecated and will be removed soon. Please use --test or --json",
+        );
+    }
+    let progress = progress || test;
+    let proxy = get_string(matches, "proxy");
+    let summary = has_flag(matches, "summary");
+    if summary {
+        logger.warning(
+            "--summary option is deprecated and will be removed soon. Please use --test or --json",
+        );
+    }
+    let summary = summary || test;
+    let timeout = match get_string(matches, "max_time") {
         None => ClientOptions::default().timeout,
         Some(s) => match s.parse::<u64>() {
             Ok(n) => Duration::from_secs(n),
@@ -408,10 +413,12 @@ pub fn parse_options(matches: ArgMatches) -> Result<CliOptions, CliError> {
             }
         },
     };
-    let to_entry = to_entry(matches.clone())?;
-    let user = matches.value_of("user").map(|x| x.to_string());
+    let to_entry = to_entry(matches)?;
+    let user = get_string(matches, "user");
+    let user_agent = get_string(matches, "user_agent");
     let variables = variables(matches.clone())?;
-    let verbose = matches.is_present("verbose") || matches.is_present("interactive");
+    let very_verbose = has_flag(matches, "very_verbose");
+    let verbose = has_flag(matches, "verbose") || has_flag(matches, "interactive") || very_verbose;
 
     Ok(CliOptions {
         cacert_file,
@@ -440,23 +447,31 @@ pub fn parse_options(matches: ArgMatches) -> Result<CliOptions, CliError> {
         timeout,
         to_entry,
         user,
+        user_agent,
         variables,
         verbose,
+        very_verbose,
     })
 }
 
-pub fn output_color(matches: ArgMatches) -> bool {
-    if matches.is_present("color") {
-        true
-    } else if matches.is_present("no_color") {
-        false
-    } else {
-        atty::is(Stream::Stdout)
+/// Returns true if Hurl output uses ANSI code and false otherwise.
+pub fn output_color(matches: &ArgMatches) -> bool {
+    if has_flag(matches, "color") {
+        return true;
     }
+    if has_flag(matches, "no_color") {
+        return false;
+    }
+    if let Ok(v) = env::var("NO_COLOR") {
+        if !v.is_empty() {
+            return false;
+        }
+    }
+    atty::is(Stream::Stdout)
 }
 
-fn to_entry(matches: ArgMatches) -> Result<Option<usize>, CliError> {
-    match matches.value_of("to_entry") {
+fn to_entry(matches: &ArgMatches) -> Result<Option<usize>, CliError> {
+    match get_string(matches, "to_entry") {
         Some(value) => match value.parse() {
             Ok(v) => Ok(Some(v)),
             Err(_) => Err(CliError {
@@ -472,15 +487,15 @@ fn variables(matches: ArgMatches) -> Result<HashMap<String, Value>, CliError> {
     let mut variables = HashMap::new();
 
     // use environment variables prefix by HURL_
-    for (env_name, env_value) in std::env::vars() {
+    for (env_name, env_value) in env::vars() {
         if let Some(name) = env_name.strip_prefix("HURL_") {
             let value = cli::parse_variable_value(env_value.as_str())?;
             variables.insert(name.to_string(), value);
         }
     }
 
-    if let Some(filename) = matches.value_of("variables_file") {
-        let path = std::path::Path::new(filename);
+    if let Some(filename) = get_string(&matches, "variables_file") {
+        let path = Path::new(&filename);
         if !path.exists() {
             return Err(CliError {
                 message: format!("Properties file {} does not exist", path.display()),
@@ -507,10 +522,9 @@ fn variables(matches: ArgMatches) -> Result<HashMap<String, Value>, CliError> {
         }
     }
 
-    if matches.is_present("variable") {
-        let input: Vec<_> = matches.values_of("variable").unwrap().collect();
+    if let Some(input) = get_strings(&matches, "variable") {
         for s in input {
-            let (name, value) = cli::parse_variable(s)?;
+            let (name, value) = cli::parse_variable(&s)?;
             variables.insert(name.to_string(), value);
         }
     }
@@ -518,13 +532,17 @@ fn variables(matches: ArgMatches) -> Result<HashMap<String, Value>, CliError> {
     Ok(variables)
 }
 
-pub fn match_glob_files(matches: ArgMatches) -> Result<Vec<String>, CliError> {
+///
+/// Returns a list of path names that match `matches`.
+///
+/// # Arguments
+/// * `matches` - A pattern to be matched
+///
+fn match_glob_files(matches: &ArgMatches) -> Result<Vec<String>, CliError> {
     let mut filenames = vec![];
-    if matches.is_present("glob") {
-        let exprs: Vec<&str> = matches.values_of("glob").unwrap().collect();
+    if let Some(exprs) = get_strings(matches, "glob") {
         for expr in exprs {
-            eprintln!("expr={}", expr);
-            let paths = match glob::glob(expr) {
+            let paths = match glob::glob(&expr) {
                 Ok(paths) => paths,
                 Err(_) => {
                     return Err(CliError {
@@ -553,3 +571,17 @@ pub fn match_glob_files(matches: ArgMatches) -> Result<Vec<String>, CliError> {
     }
     Ok(filenames)
 }
+
+fn get_string(matches: &ArgMatches, name: &str) -> Option<String> {
+    matches.get_one::<String>(name).map(|x| x.to_string())
+}
+
+pub fn get_strings(matches: &ArgMatches, name: &str) -> Option<Vec<String>> {
+    matches
+        .get_many::<String>(name)
+        .map(|v| v.map(|x| x.to_string()).collect())
+}
+
+pub fn has_flag(matches: &ArgMatches, name: &str) -> bool {
+    matches.contains_id(name)
+}
