@@ -19,12 +19,6 @@
 use core::fmt;
 use std::str::FromStr;
 
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct Header {
-    pub name: String,
-    pub value: String,
-}
-
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Cookie {
     pub domain: String,
@@ -49,13 +43,30 @@ pub struct Param {
     pub value: String,
 }
 
-impl fmt::Display for Header {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}: {}", self.name, self.value)
-    }
-}
-
 impl fmt::Display for Cookie {
+    /// Formats this cookie using Netscape cookie format.
+    ///
+    /// http://www.cookiecentral.com/faq/#3.5
+    ///
+    /// > The layout of Netscape's cookies.txt file is such that each line contains one name-value
+    /// > pair. An example cookies.txt file may have an entry that looks like this:
+    /// >
+    /// > `.netscape.com     TRUE   /  FALSE  946684799   NETSCAPE_ID  100103`
+    /// >
+    /// > Each line represents a single piece of stored information. A tab is inserted between each
+    /// > of the fields.
+    /// > From left-to-right, here is what each field represents:
+    /// > - domain - The domain that created AND that can read the variable.
+    /// > - flag - A TRUE/FALSE value indicating if all machines within a given domain can access
+    /// > the variable. This value is set automatically by the browser, depending on the value you
+    /// > set for domain.
+    /// > - path - The path within the domain that the variable is valid for.
+    /// > - secure - A TRUE/FALSE value indicating if a secure connection with the domain is
+    /// > needed to access the variable.
+    /// > - expiration - The UNIX time that the variable will expire on. UNIX time is defined as the
+    /// > - number of seconds since Jan 1, 1970 00:00:00 GMT.
+    /// > - name - The name of the variable.
+    /// > - value - The value of the variable.
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
             f,
@@ -92,7 +103,7 @@ impl FromStr for Cookie {
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let tokens = s.split_ascii_whitespace().collect::<Vec<&str>>();
-        let (http_only, domain) = if let Some(&v) = tokens.get(0) {
+        let (http_only, domain) = if let Some(&v) = tokens.first() {
             if let Some(domain) = v.strip_prefix("#HttpOnly_") {
                 (true, domain.to_string())
             } else {
@@ -144,22 +155,6 @@ impl FromStr for Cookie {
     }
 }
 
-///
-/// Return a list of headers values for the given header name.
-///
-pub fn get_header_values(headers: &[Header], expected_name: &str) -> Vec<String> {
-    headers
-        .iter()
-        .filter_map(|Header { name, value }| {
-            if name == expected_name {
-                Some(value.to_string())
-            } else {
-                None
-            }
-        })
-        .collect()
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
