@@ -25,18 +25,21 @@ use atty::Stream;
 use colored::*;
 
 use hurl::cli;
-use hurl::cli::{CliError, CliOptions, OutputType};
+use hurl::cli::{CliError, CliOptions, Logger, OutputType};
 use hurl::http;
+use hurl::http::ContextDir;
 use hurl::report;
+use hurl::report::canonicalize_filename;
 use hurl::runner;
-use hurl::runner::{HurlResult, RunnerOptions};
-use hurl_core::ast::{Pos, SourceInfo};
+use hurl::runner::{HurlResult, RunnerError, RunnerOptions, Verbosity};
+use hurl::util::logger::BaseLogger;
+use hurl_core::ast::{Entry, Pos, SourceInfo};
 use hurl_core::error::Error;
 use hurl_core::parser;
 
 #[cfg(target_family = "unix")]
 pub fn init_colored() {
-    colored::control::set_override(true);
+    control::set_override(true);
 }
 
 const EXIT_OK: i32 = 0;
@@ -81,83 +84,74 @@ struct Progress {
     pub total: usize,
 }
 
+/// Runs a Hurl file and returns a result.
+///
+/// # Arguments
+///
+/// * `filename` - Filename of the Hurl file, "-" is used for stdin
+/// * `content` - Content of the Hurl file
+/// * `current_dir` - The current directory of execution (absolute)
+/// * `cli_options` - Options for this run
+/// * `progress` - The progression of this execution
+/// * `logger` - The logger
 fn execute(
     filename: &str,
-    contents: String,
+    content: &str,
     current_dir: &Path,
-    cli_options: CliOptions,
-    log_verbose: &impl Fn(&str),
-    log_error_message: &impl Fn(bool, &str),
-    progress: Option<Progress>,
+    cli_options: &CliOptions,
+    progress: &Option<Progress>,
+    logger: &Logger,
 ) -> HurlResult {
-    let lines: Vec<String> = regex::Regex::new(r"\n|\r\n")
-        .unwrap()
-        .split(&contents)
-        .map(|l| l.to_string())
-        .collect();
-    let optional_filename = if filename.is_empty() {
-        None
-    } else {
-        Some(filename.to_string())
-    };
-
     if let Some(Progress { current, total }) = progress {
-        eprintln!("{}: RUNNING [{}/{}]", filename, current + 1, total);
+        logger.test_running(current + 1, *total);
     }
-    let log_parser_error =
-        cli::make_logger_parser_error(lines.clone(), cli_options.color, optional_filename.clone());
-    let log_runner_error =
-        cli::make_logger_runner_error(lines, cli_options.color, optional_filename);
 
-    match parser::parse_hurl_file(contents.as_str()) {
+    match parser::parse_hurl_file(content) {
         Err(e) => {
-            log_parser_error(&e, false);
+            logger.error_rich(&e);
             std::process::exit(EXIT_ERROR_PARSING);
         }
         Ok(hurl_file) => {
-            log_verbose(format!("fail fast: {}", cli_options.fail_fast).as_str());
-            log_verbose(format!("insecure: {}", cli_options.insecure).as_str());
-            log_verbose(format!("follow redirect: {}", cli_options.follow_location).as_str());
+            logger.debug_important("Options:");
+            logger.debug(format!("    fail fast: {}", cli_options.fail_fast).as_str());
+            logger.debug(format!("    insecure: {}", cli_options.insecure).as_str());
+            logger.debug(format!("    follow redirect: {}", cli_options.follow_location).as_str());
             if let Some(n) = cli_options.max_redirect {
-                log_verbose(format!("max redirect: {}", n).as_str());
+                logger.debug(format!("    max redirect: {}", n).as_str());
             }
-            if let Some(proxy) = cli_options.proxy.clone() {
-                log_verbose(format!("proxy: {}", proxy).as_str());
+            if let Some(proxy) = &cli_options.proxy {
+                logger.debug(format!("    proxy: {}", proxy).as_str());
             }
-
             if !cli_options.variables.is_empty() {
-                log_verbose("variables:");
+                logger.debug_important("Variables:");
                 for (name, value) in cli_options.variables.clone() {
-                    log_verbose(format!("    {}={}", name, value).as_str());
+                    logger.debug(format!("    {}: {}", name, value).as_str());
                 }
             }
-
             if let Some(to_entry) = cli_options.to_entry {
-                if to_entry < hurl_file.entries.len() {
-                    log_verbose(
-                        format!("executing {}/{} entries", to_entry, hurl_file.entries.len())
-                            .as_str(),
-                    );
-                } else {
-                    log_verbose("executing all entries");
-                }
+                logger.debug(
+                    format!("Executing {}/{} entries", to_entry, hurl_file.entries.len()).as_str(),
+                );
             }
-
-            let cacert_file = cli_options.cacert_file;
+            let cacert_file = cli_options.cacert_file.clone();
             let follow_location = cli_options.follow_location;
-            let verbose = cli_options.verbose;
+            let verbosity = match (cli_options.verbose, cli_options.very_verbose) {
+                (true, true) => Some(Verbosity::VeryVerbose),
+                (true, _) => Some(Verbosity::Verbose),
+                _ => None,
+            };
             let insecure = cli_options.insecure;
             let max_redirect = cli_options.max_redirect;
-            let proxy = cli_options.proxy;
-            let no_proxy = cli_options.no_proxy;
-            let cookie_input_file = cli_options.cookie_input_file;
-
+            let proxy = cli_options.proxy.clone();
+            let no_proxy = cli_options.no_proxy.clone();
+            let cookie_input_file = cli_options.cookie_input_file.clone();
             let timeout = cli_options.timeout;
             let connect_timeout = cli_options.connect_timeout;
-            let user = cli_options.user;
-            let user_agent = cli_options.user_agent;
+            let user = cli_options.user.clone();
+            let user_agent = cli_options.user_agent.clone();
             let compressed = cli_options.compressed;
-            let context_dir = match cli_options.file_root {
+            let file_root = match cli_options.file_root {
+                Some(ref filename) => Path::new(filename),
                 None => {
                     if filename == "-" {
                         current_dir
@@ -166,82 +160,84 @@ fn execute(
                         path.parent().unwrap()
                     }
                 }
-                Some(ref filename) => Path::new(filename),
             };
-            let options = http::ClientOptions {
-                cacert_file,
-                follow_location,
-                max_redirect,
-                cookie_input_file,
-                proxy,
-                no_proxy,
-                verbose,
-                insecure,
-                timeout,
-                connect_timeout,
-                user,
-                user_agent,
-                compressed,
-                context_dir: context_dir.to_path_buf(),
-            };
-
-            let mut client = http::Client::init(options);
+            let context_dir = ContextDir::new(current_dir, file_root);
 
             let pre_entry = if cli_options.interactive {
-                cli::interactive::pre_entry
+                Some(cli::interactive::pre_entry as fn(Entry) -> bool)
             } else {
-                |_| false
+                None
             };
             let post_entry = if cli_options.interactive {
-                cli::interactive::post_entry
+                Some(cli::interactive::post_entry as fn() -> bool)
             } else {
-                || false
+                None
             };
-            let options = RunnerOptions {
-                fail_fast: cli_options.fail_fast,
-                variables: cli_options.variables,
-                to_entry: cli_options.to_entry,
-                context_dir: context_dir.to_path_buf(),
-                ignore_asserts: cli_options.ignore_asserts,
+            let fail_fast = cli_options.fail_fast;
+            let variables = cli_options.variables.clone();
+            let to_entry = cli_options.to_entry;
+            let ignore_asserts = cli_options.ignore_asserts;
+            let very_verbose = cli_options.very_verbose;
+            let runner_options = RunnerOptions {
+                cacert_file,
+                compressed,
+                fail_fast,
+                to_entry,
+                user,
+                context_dir,
+                ignore_asserts,
+                insecure,
+                max_redirect,
+                very_verbose,
                 pre_entry,
+                proxy,
                 post_entry,
+                connect_timeout,
+                cookie_input_file,
+                follow_location,
+                no_proxy,
+                timeout,
+                user_agent,
+                verbosity,
             };
-            let result = runner::run_hurl_file(
-                hurl_file,
+            let mut client = http::Client::new(runner_options.cookie_input_file.clone());
+            let result = runner::run(
+                &hurl_file,
+                filename,
                 &mut client,
-                filename.to_string(),
-                &options,
-                &log_verbose,
-                &log_error_message,
-                &log_runner_error,
+                &runner_options,
+                &variables,
+                logger,
             );
             if cli_options.progress {
-                let status = match (result.success, cli_options.color) {
-                    (true, true) => "SUCCESS".green().to_string(),
-                    (true, false) => "SUCCESS".to_string(),
-                    (false, true) => "FAILURE".red().to_string(),
-                    (false, false) => "FAILURE".to_string(),
-                };
-                eprintln!("{}: {}", filename, status);
+                logger.test_completed(&result);
             }
             result
         }
     }
 }
 
-pub fn unwrap_or_exit<T>(
-    log_error_message: &impl Fn(bool, &str),
-    result: Result<T, CliError>,
-) -> T {
+/// Unwraps a result or exit with message.
+///
+/// # Arguments
+///
+/// * result - Something to unwrap
+/// * logger - A logger to log the error
+fn unwrap_or_exit<T>(result: Result<T, CliError>, code: i32, logger: &BaseLogger) -> T {
     match result {
         Ok(v) => v,
-        Err(e) => {
-            log_error_message(false, e.message.as_str());
-            std::process::exit(EXIT_ERROR_UNDEFINED);
-        }
+        Err(e) => exit_with_error(&e.message, code, logger),
     }
 }
 
+/// Prints an error message and exits the current process with an exit code.
+fn exit_with_error(message: &str, code: i32, logger: &BaseLogger) -> ! {
+    if !message.is_empty() {
+        logger.error(message);
+    }
+    std::process::exit(code);
+}
+/// Executes Hurl entry point.
 fn main() {
     let version_info = format!(
         "{} {}",
@@ -252,14 +248,16 @@ fn main() {
     let matches = app.clone().get_matches();
     init_colored();
 
-    let verbose = matches.is_present("verbose") || matches.is_present("interactive");
-    let log_verbose = cli::make_logger_verbose(verbose);
-    let color = cli::output_color(matches.clone());
-    let log_error_message = cli::make_logger_error_message(color);
-    let cli_options = unwrap_or_exit(&log_error_message, cli::parse_options(matches.clone()));
+    let verbose = cli::has_flag(&matches, "verbose")
+        || cli::has_flag(&matches, "very_verbose")
+        || cli::has_flag(&matches, "interactive");
+    let color = cli::output_color(&matches);
+    let base_logger = BaseLogger::new(color, verbose);
+    let cli_options = cli::parse_options(&matches, &base_logger);
+    let cli_options = unwrap_or_exit(cli_options, EXIT_ERROR_UNDEFINED, &base_logger);
 
     let mut filenames = vec![];
-    if let Some(values) = matches.values_of("INPUT") {
+    if let Some(values) = cli::get_strings(&matches, "INPUT") {
         for value in values {
             filenames.push(value.to_string());
         }
@@ -269,42 +267,50 @@ fn main() {
     }
 
     if filenames.is_empty() && atty::is(Stream::Stdin) {
-        if app.clone().print_help().is_err() {
-            panic!("panic during printing help");
-        }
-        println!();
-        std::process::exit(EXIT_ERROR_COMMANDLINE);
+        let error = if app.clone().print_help().is_err() {
+            "Panic during printing help"
+        } else {
+            ""
+        };
+        exit_with_error(error, EXIT_ERROR_COMMANDLINE, &base_logger);
     } else if filenames.is_empty() {
         filenames.push("-".to_string());
     }
 
-    let current_dir_buf = std::env::current_dir().unwrap();
-    let current_dir = current_dir_buf.as_path();
-
-    let mut hurl_results = vec![];
-
+    let current_dir = match std::env::current_dir() {
+        Ok(c) => c,
+        Err(error) => {
+            exit_with_error(
+                error.to_string().as_str(),
+                EXIT_ERROR_UNDEFINED,
+                &base_logger,
+            );
+        }
+    };
+    let current_dir = current_dir.as_path();
     let cookies_output_file = match cli_options.cookie_output_file.clone() {
         None => None,
         Some(filename) => {
-            let filename = unwrap_or_exit(
-                &log_error_message,
-                cookies_output_file(filename, filenames.len()),
-            );
+            let result = cookies_output_file(&filename, filenames.len());
+            let filename = unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
             Some(filename)
         }
     };
 
     let start = Instant::now();
+    let mut hurl_results = vec![];
     let mut testcases = vec![];
 
     for (current, filename) in filenames.iter().enumerate() {
-        let contents = match cli::read_to_string(filename) {
-            Ok(v) => v,
-            Err(e) => {
-                log_error_message(false, e.message.as_str());
-                std::process::exit(EXIT_ERROR_PARSING);
-            }
-        };
+        if filename != "-" && !Path::new(filename).exists() {
+            let message = format!(
+                "hurl: cannot access '{}': No such file or directory",
+                filename
+            );
+            exit_with_error(&message, EXIT_ERROR_PARSING, &base_logger);
+        }
+        let content = cli::read_to_string(filename);
+        let content = unwrap_or_exit(content, EXIT_ERROR_PARSING, &base_logger);
 
         let progress = if cli_options.progress {
             Some(Progress {
@@ -314,14 +320,15 @@ fn main() {
         } else {
             None
         };
+        let logger = Logger::new(color, verbose, filename, &content);
+
         let hurl_result = execute(
             filename,
-            contents.clone(),
+            &content,
             current_dir,
-            cli_options.clone(),
-            &log_verbose,
-            &log_error_message,
-            progress,
+            &cli_options,
+            &progress,
+            &logger,
         );
         hurl_results.push(hurl_result.clone());
 
@@ -329,11 +336,13 @@ fn main() {
             && hurl_result.errors().is_empty()
             && !cli_options.interactive
         {
-            // default
-            // last entry + response + body
+            // By default, we output the body response bytes of the last entry
             if let Some(entry_result) = hurl_result.entries.last() {
-                if let Some(response) = entry_result.response.clone() {
+                if let Some(response) = &entry_result.response {
                     let mut output = vec![];
+
+                    // If include options is set, we output the HTTP response headers
+                    // with status and version (to mimic curl outputs)
                     if cli_options.include {
                         let status_line =
                             format!("HTTP/{} {}\n", response.version, response.status);
@@ -344,36 +353,33 @@ fn main() {
                         }
                         output.append(&mut "\n".to_string().into_bytes());
                     }
-                    let body = if cli_options.compressed {
+                    let mut body = if entry_result.compressed {
                         match response.uncompress_body() {
                             Ok(bytes) => bytes,
                             Err(e) => {
-                                log_error_message(
-                                    false,
-                                    runner::Error {
-                                        source_info: SourceInfo {
-                                            start: Pos { line: 0, column: 0 },
-                                            end: Pos { line: 0, column: 0 },
-                                        },
-                                        inner: e,
-                                        assert: false,
-                                    }
-                                    .fixme()
-                                    .as_str(),
-                                );
-                                std::process::exit(EXIT_ERROR_RUNTIME);
+                                // FIXME: we convert to a runner::Error to be able to use fixme
+                                // method. Can we do otherwise (without creating an artificial
+                                // error a first character).
+                                let error = runner::Error {
+                                    source_info: SourceInfo {
+                                        start: Pos { line: 0, column: 0 },
+                                        end: Pos { line: 0, column: 0 },
+                                    },
+                                    inner: RunnerError::from(e),
+                                    assert: false,
+                                };
+                                let message = error.fixme();
+                                exit_with_error(&message, EXIT_ERROR_RUNTIME, &base_logger);
                             }
                         }
                     } else {
-                        response.body
+                        response.body.clone()
                     };
-                    output.append(&mut body.clone());
-                    unwrap_or_exit(
-                        &log_error_message,
-                        write_output(output, cli_options.output.clone()),
-                    );
+                    output.append(&mut body);
+                    let result = write_output(&output, &cli_options.output);
+                    unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
                 } else {
-                    cli::log_info("no response has been received");
+                    logger.info("No response has been received");
                 }
             } else {
                 let source = if filename.as_str() == "-" {
@@ -381,80 +387,63 @@ fn main() {
                 } else {
                     format!("for file {}", filename).to_string()
                 };
-                log_error_message(
-                    true,
-                    format!("no entry have been executed {}", source).as_str(),
-                );
+                logger.warning(format!("No entry have been executed {}", source).as_str());
             };
         }
 
-        let lines: Vec<String> = regex::Regex::new(r"\n|\r\n")
-            .unwrap()
-            .split(&contents)
-            .map(|l| l.to_string())
-            .collect();
         if matches!(cli_options.output_type, OutputType::Json) {
-            let json_result = hurl_result.to_json(&lines);
+            let json_result = hurl_result.to_json(&content);
             let serialized = serde_json::to_string(&json_result).unwrap();
             let s = format!("{}\n", serialized);
-            unwrap_or_exit(
-                &log_error_message,
-                write_output(s.into_bytes(), cli_options.output.clone()),
-            );
+            let result = write_output(&s.into_bytes(), &cli_options.output);
+            unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
         }
         if cli_options.junit_file.is_some() {
-            let testcase = report::Testcase::from_hurl_result(&hurl_result, &lines);
+            let testcase = report::Testcase::from_hurl_result(&hurl_result, &content);
             testcases.push(testcase);
         }
     }
 
     if let Some(filename) = cli_options.junit_file.clone() {
-        log_verbose(format!("Writing Junit report to {}", filename).as_str());
-        unwrap_or_exit(
-            &log_error_message,
-            report::create_junit_report(filename, testcases),
-        );
+        base_logger.debug(format!("Writing Junit report to {}", filename).as_str());
+        let result = report::create_junit_report(filename, testcases);
+        unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
     }
 
     if let Some(dir_path) = cli_options.html_dir {
-        log_verbose(format!("Writing html report to {}", dir_path.display()).as_str());
-        unwrap_or_exit(
-            &log_error_message,
-            report::write_html_report(dir_path.clone(), hurl_results.clone()),
-        );
+        base_logger.debug(format!("Writing html report to {}", dir_path.display()).as_str());
+        let result = report::write_html_report(dir_path.clone(), hurl_results.clone());
+        unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
 
         for filename in filenames {
-            unwrap_or_exit(
-                &log_error_message,
-                format_html(filename.as_str(), dir_path.clone()),
-            );
+            let result = format_html(filename.as_str(), &dir_path);
+            unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
         }
     }
 
     if let Some(file_path) = cookies_output_file {
-        log_verbose(format!("Writing cookies to {}", file_path.display()).as_str());
-        unwrap_or_exit(
-            &log_error_message,
-            write_cookies_file(file_path, hurl_results.clone()),
-        );
+        base_logger.debug(format!("Writing cookies to {}", file_path.display()).as_str());
+        let result = write_cookies_file(&file_path, &hurl_results);
+        unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
     }
 
     if cli_options.summary {
         let duration = start.elapsed().as_millis();
-        let summary = get_summary(duration, hurl_results.clone());
-        eprintln!("{}", summary.as_str());
+        let summary = get_summary(duration, &hurl_results);
+        base_logger.info(summary.as_str());
     }
 
-    std::process::exit(exit_code(hurl_results));
+    std::process::exit(exit_code(&hurl_results));
 }
 
-fn exit_code(hurl_results: Vec<HurlResult>) -> i32 {
+/// Returns an exit code for a list of HurlResult.
+fn exit_code(hurl_results: &[HurlResult]) -> i32 {
     let mut count_errors_runner = 0;
     let mut count_errors_assert = 0;
     for hurl_result in hurl_results {
         let errors = hurl_result.clone().errors();
         if errors.is_empty() {
-        } else if errors.iter().filter(|e| !e.assert).cloned().count() == 0 {
+        } else if errors.iter().filter(|e| !e.assert).count() == 0 {
             count_errors_assert += 1;
         } else {
             count_errors_runner += 1;
@@ -469,14 +458,20 @@ fn exit_code(hurl_results: Vec<HurlResult>) -> i32 {
     }
 }
 
-fn format_html(input_file: &str, dir_path: PathBuf) -> Result<(), CliError> {
-    let file_path = dir_path.join(format!("{}.html", input_file));
-    let parent = file_path.parent().expect("a parent");
+fn format_html(input_file: &str, dir_path: &Path) -> Result<(), CliError> {
+    let relative_input_file = canonicalize_filename(input_file);
+    let absolute_input_file = dir_path.join(format!("{}.html", relative_input_file));
+
+    let parent = absolute_input_file.parent().expect("a parent");
     std::fs::create_dir_all(parent).unwrap();
-    let mut file = match std::fs::File::create(&file_path) {
+    let mut file = match std::fs::File::create(&absolute_input_file) {
         Err(why) => {
             return Err(CliError {
-                message: format!("Issue writing to {}: {:?}", file_path.display(), why),
+                message: format!(
+                    "Issue writing to {}: {:?}",
+                    absolute_input_file.display(),
+                    why
+                ),
             });
         }
         Ok(file) => file,
@@ -488,13 +483,17 @@ fn format_html(input_file: &str, dir_path: PathBuf) -> Result<(), CliError> {
 
     if let Err(why) = file.write_all(s.as_bytes()) {
         return Err(CliError {
-            message: format!("Issue writing to {}: {:?}", file_path.display(), why),
+            message: format!(
+                "Issue writing to {}: {:?}",
+                absolute_input_file.display(),
+                why
+            ),
         });
     }
     Ok(())
 }
 
-fn write_output(bytes: Vec<u8>, filename: Option<String>) -> Result<(), CliError> {
+fn write_output(bytes: &Vec<u8>, filename: &Option<String>) -> Result<(), CliError> {
     match filename {
         None => write_bytes(bytes.as_slice()),
         Some(filename) => {
@@ -514,18 +513,18 @@ fn write_output(bytes: Vec<u8>, filename: Option<String>) -> Result<(), CliError
     }
 }
 
-pub fn cookies_output_file(filename: String, n: usize) -> Result<std::path::PathBuf, CliError> {
+fn cookies_output_file(filename: &str, n: usize) -> Result<PathBuf, CliError> {
     if n > 1 {
         Err(CliError {
             message: "Only save cookies for a unique session".to_string(),
         })
     } else {
-        let path = std::path::Path::new(&filename);
+        let path = Path::new(&filename);
         Ok(path.to_path_buf())
     }
 }
 
-fn write_cookies_file(file_path: PathBuf, hurl_results: Vec<HurlResult>) -> Result<(), CliError> {
+fn write_cookies_file(file_path: &Path, hurl_results: &[HurlResult]) -> Result<(), CliError> {
     let mut file = match std::fs::File::create(&file_path) {
         Err(why) => {
             return Err(CliError {
@@ -561,17 +560,17 @@ fn write_cookies_file(file_path: PathBuf, hurl_results: Vec<HurlResult>) -> Resu
     Ok(())
 }
 
-fn get_summary(duration: u128, hurl_results: Vec<HurlResult>) -> String {
+fn get_summary(duration: u128, hurl_results: &[HurlResult]) -> String {
     let total = hurl_results.len();
     let success = hurl_results.iter().filter(|r| r.success).count();
     let failed = total - success;
     let mut s =
         "--------------------------------------------------------------------------------\n"
             .to_string();
-    s.push_str(format!("Executed:  {}\n", total).as_str());
+    s.push_str(format!("Executed files:  {}\n", total).as_str());
     s.push_str(
         format!(
-            "Succeeded: {} ({:.1}%)\n",
+            "Succeeded files: {} ({:.1}%)\n",
             success,
             100.0 * success as f32 / total as f32
         )
@@ -579,12 +578,12 @@ fn get_summary(duration: u128, hurl_results: Vec<HurlResult>) -> String {
     );
     s.push_str(
         format!(
-            "Failed:    {} ({:.1}%)\n",
+            "Failed files:    {} ({:.1}%)\n",
             failed,
             100.0 * failed as f32 / total as f32
         )
         .as_str(),
     );
-    s.push_str(format!("Duration:  {}ms\n", duration).as_str());
+    s.push_str(format!("Duration:        {} ms\n", duration).as_str());
     s
 }
