@@ -26,24 +26,29 @@ use super::core::{CaptureResult, Error};
 use super::query::eval_query;
 use super::value::Value;
 
+/// Evaluates a `capture` with `variables` map and `http_response`, returns a
+/// [`CaptureResult`] on success or an [`Error`] .
 pub fn eval_capture(
-    capture: Capture,
+    capture: &Capture,
     variables: &HashMap<String, Value>,
-    http_response: http::Response,
+    http_response: &http::Response,
 ) -> Result<CaptureResult, Error> {
-    let name = capture.name.value;
-    let value = eval_query(capture.query.clone(), variables, http_response)?;
+    let name = &capture.name.value;
+    let value = eval_query(&capture.query, variables, http_response)?;
     let value = match value {
         None => {
             return Err(Error {
-                source_info: capture.query.source_info,
+                source_info: capture.query.source_info.clone(),
                 inner: RunnerError::NoQueryResult {},
                 assert: false,
             });
         }
         Some(value) => value,
     };
-    Ok(CaptureResult { name, value })
+    Ok(CaptureResult {
+        name: name.clone(),
+        value,
+    })
 }
 
 #[cfg(test)]
@@ -58,7 +63,7 @@ pub mod tests {
         // non scalar value
         let whitespace = Whitespace {
             value: String::from(""),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         Capture {
             line_terminators: vec![],
@@ -67,7 +72,7 @@ pub mod tests {
                 quotes: false,
                 value: "UserCount".to_string(),
                 encoded: "UserCount".to_string(),
-                source_info: SourceInfo::init(0, 0, 0, 0),
+                source_info: SourceInfo::new(0, 0, 0, 0),
             },
             space1: whitespace.clone(),
             space2: whitespace.clone(),
@@ -86,7 +91,7 @@ pub mod tests {
         // non scalar value
         let whitespace = Whitespace {
             value: String::from(""),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         Capture {
             line_terminators: vec![],
@@ -95,7 +100,7 @@ pub mod tests {
                 quotes: false,
                 value: "duration".to_string(),
                 encoded: "duration".to_string(),
-                source_info: SourceInfo::init(0, 0, 0, 0),
+                source_info: SourceInfo::new(0, 0, 0, 0),
             },
             space1: whitespace.clone(),
             space2: whitespace.clone(),
@@ -115,7 +120,7 @@ pub mod tests {
         let variables = HashMap::new();
         let whitespace = Whitespace {
             value: String::from(""),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         let capture = Capture {
             line_terminators: vec![],
@@ -124,7 +129,7 @@ pub mod tests {
                 quotes: false,
                 value: "count".to_string(),
                 encoded: "count".to_string(),
-                source_info: SourceInfo::init(0, 0, 0, 0),
+                source_info: SourceInfo::new(0, 0, 0, 0),
             },
             space1: whitespace.clone(),
             space2: whitespace.clone(),
@@ -137,7 +142,7 @@ pub mod tests {
             },
         };
 
-        let error = eval_capture(capture, &variables, http::xml_three_users_http_response())
+        let error = eval_capture(&capture, &variables, &http::xml_three_users_http_response())
             .err()
             .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 7 });
@@ -149,7 +154,7 @@ pub mod tests {
         // non scalar value
         let whitespace = Whitespace {
             value: String::from(""),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         let _capture = Capture {
             line_terminators: vec![],
@@ -158,14 +163,14 @@ pub mod tests {
                 quotes: false,
                 value: "???".to_string(),
                 encoded: "???".to_string(),
-                source_info: SourceInfo::init(0, 0, 0, 0),
+                source_info: SourceInfo::new(0, 0, 0, 0),
             },
             space1: whitespace.clone(),
             space2: whitespace.clone(),
 
             // xpath //user
             query: Query {
-                source_info: SourceInfo::init(1, 1, 1, 13),
+                source_info: SourceInfo::new(1, 1, 1, 13),
                 value: QueryValue::Xpath {
                     space0: whitespace.clone(),
                     expr: Template {
@@ -174,7 +179,7 @@ pub mod tests {
                             value: "//user".to_string(),
                             encoded: "//user".to_string(),
                         }],
-                        source_info: SourceInfo::init(1, 7, 1, 13),
+                        source_info: SourceInfo::new(1, 7, 1, 13),
                     },
                 },
 
@@ -193,9 +198,9 @@ pub mod tests {
         let variables = HashMap::new();
         assert_eq!(
             eval_capture(
-                user_count_capture(),
+                &user_count_capture(),
                 &variables,
-                http::xml_three_users_http_response(),
+                &http::xml_three_users_http_response(),
             )
             .unwrap(),
             CaptureResult {
@@ -205,7 +210,7 @@ pub mod tests {
         );
 
         assert_eq!(
-            eval_capture(duration_capture(), &variables, http::json_http_response()).unwrap(),
+            eval_capture(&duration_capture(), &variables, &http::json_http_response()).unwrap(),
             CaptureResult {
                 name: "duration".to_string(),
                 value: Value::from_f64(1.5),
