@@ -1,6 +1,6 @@
 /*
- * hurl (https://hurl.dev)
- * Copyright (C) 2020 Orange
+ * Hurl (https://hurl.dev)
+ * Copyright (C) 2022 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,25 +23,22 @@
 //!
 //!
 
-pub use self::core::{Error, HurlResult, RunnerError, RunnerOptions};
-pub use self::hurl_file::run as run_hurl_file;
-pub use self::log_deserialize::parse_results as deserialize_results;
+pub use self::core::{
+    AssertResult, CaptureResult, EntryResult, Error, HurlResult, RunnerError, RunnerOptions,
+    Verbosity,
+};
+pub use self::hurl_file::run;
 pub use self::value::Value;
 
 mod assert;
 mod body;
 mod capture;
-mod content_decoding;
-mod cookie;
 mod core;
 mod entry;
 mod error;
 mod expr;
-mod http_response;
 mod hurl_file;
 mod json;
-mod log_deserialize;
-mod log_serialize;
 mod multipart;
 mod predicate;
 mod predicate_value;
