@@ -19,7 +19,6 @@
 use regex::Regex;
 use std::collections::HashMap;
 
-use super::cookie;
 use super::core::{Error, RunnerError};
 use super::template::eval_template;
 use super::value::Value;
@@ -33,18 +32,18 @@ use sha2::Digest;
 pub type QueryResult = Result<Option<Value>, Error>;
 
 pub fn eval_query(
-    query: Query,
+    query: &Query,
     variables: &HashMap<String, Value>,
-    http_response: http::Response,
+    http_response: &http::Response,
 ) -> QueryResult {
-    let value = eval_query_value(query.clone(), variables, http_response)?;
-    if let Some((_, subquery)) = query.subquery {
-        if let Some(value) = value {
+    let value = eval_query_value(query, variables, http_response)?;
+    if let Some((_, subquery)) = &query.subquery {
+        if let Some(value) = &value {
             eval_subquery(subquery, value, variables)
         } else {
             Err(Error {
-                source_info: subquery.source_info,
-                inner: RunnerError::SubqueryInvalidInput,
+                source_info: subquery.source_info.clone(),
+                inner: RunnerError::SubqueryInvalidInput("none".to_string()),
                 assert: false,
             })
         }
@@ -53,15 +52,15 @@ pub fn eval_query(
     }
 }
 pub fn eval_query_value(
-    query: Query,
+    query: &Query,
     variables: &HashMap<String, Value>,
-    http_response: http::Response,
+    http_response: &http::Response,
 ) -> QueryResult {
     match query.value.clone() {
         QueryValue::Status {} => Ok(Some(Value::Integer(i64::from(http_response.status)))),
         QueryValue::Header { name, .. } => {
             let header_name = eval_template(&name, variables)?;
-            let values = http_response.get_header(header_name);
+            let values = http_response.get_header_values(&header_name);
             if values.is_empty() {
                 Ok(None)
             } else if values.len() == 1 {
@@ -97,41 +96,41 @@ pub fn eval_query_value(
             match http_response.text() {
                 Ok(s) => Ok(Some(Value::String(s))),
                 Err(inner) => Err(Error {
-                    source_info: query.source_info,
-                    inner,
+                    source_info: query.source_info.clone(),
+                    inner: RunnerError::from(inner),
                     assert: false,
                 }),
             }
         }
         QueryValue::Xpath { expr, .. } => {
-            let source_info = expr.source_info.clone();
+            let source_info = &expr.source_info;
             let value = eval_template(&expr, variables)?;
             match http_response.text() {
                 Err(inner) => Err(Error {
-                    source_info: query.source_info,
-                    inner,
+                    source_info: query.source_info.clone(),
+                    inner: RunnerError::from(inner),
                     assert: false,
                 }),
                 Ok(xml) => {
                     let result = if http_response.is_html() {
-                        xpath::eval_html(xml, value.clone())
+                        xpath::eval_html(&xml, &value)
                     } else {
-                        xpath::eval_xml(xml, value.clone())
+                        xpath::eval_xml(&xml, &value)
                     };
                     match result {
                         Ok(value) => Ok(Some(value)),
                         Err(xpath::XpathError::InvalidXml {}) => Err(Error {
-                            source_info: query.source_info,
+                            source_info: query.source_info.clone(),
                             inner: RunnerError::QueryInvalidXml,
                             assert: false,
                         }),
                         Err(xpath::XpathError::InvalidHtml {}) => Err(Error {
-                            source_info: query.source_info,
+                            source_info: query.source_info.clone(),
                             inner: RunnerError::QueryInvalidXml,
                             assert: false,
                         }),
                         Err(xpath::XpathError::Eval {}) => Err(Error {
-                            source_info,
+                            source_info: source_info.clone(),
                             inner: RunnerError::QueryInvalidXpathEval,
                             assert: false,
                         }),
@@ -158,8 +157,8 @@ pub fn eval_query_value(
             let json = match http_response.text() {
                 Err(inner) => {
                     return Err(Error {
-                        source_info: query.source_info,
-                        inner,
+                        source_info: query.source_info.clone(),
+                        inner: RunnerError::from(inner),
                         assert: false,
                     });
                 }
@@ -168,7 +167,7 @@ pub fn eval_query_value(
             let value = match serde_json::from_str(json.as_str()) {
                 Err(_) => {
                     return Err(Error {
-                        source_info: query.source_info,
+                        source_info: query.source_info.clone(),
                         inner: RunnerError::QueryInvalidJson,
                         assert: false,
                     });
@@ -185,32 +184,40 @@ pub fn eval_query_value(
                 Ok(Some(Value::from_json(&serde_json::Value::Array(results))))
             }
         }
-        QueryValue::Regex { expr, .. } => {
-            let value = eval_template(&expr, variables)?;
-            let source_info = expr.source_info;
+        QueryValue::Regex { value, .. } => {
             let s = match http_response.text() {
                 Err(inner) => {
                     return Err(Error {
-                        source_info: query.source_info,
-                        inner,
+                        source_info: query.source_info.clone(),
+                        inner: RunnerError::from(inner),
                         assert: false,
                     });
                 }
                 Ok(v) => v,
             };
-            match Regex::new(value.as_str()) {
-                Ok(re) => match re.captures(s.as_str()) {
-                    Some(captures) => match captures.get(1) {
-                        Some(v) => Ok(Some(Value::String(v.as_str().to_string()))),
-                        None => Ok(None),
-                    },
+            let re = match value {
+                RegexValue::Template(t) => {
+                    let value = eval_template(&t, variables)?;
+                    match Regex::new(value.as_str()) {
+                        Ok(re) => re,
+                        Err(_) => {
+                            let source_info = t.source_info;
+                            return Err(Error {
+                                source_info,
+                                inner: RunnerError::InvalidRegex(),
+                                assert: false,
+                            });
+                        }
+                    }
+                }
+                RegexValue::Regex(re) => re.inner,
+            };
+            match re.captures(s.as_str()) {
+                Some(captures) => match captures.get(1) {
+                    Some(v) => Ok(Some(Value::String(v.as_str().to_string()))),
                     None => Ok(None),
                 },
-                Err(_) => Err(Error {
-                    source_info,
-                    inner: RunnerError::InvalidRegex(),
-                    assert: false,
-                }),
+                None => Ok(None),
             }
         }
         QueryValue::Variable { name, .. } => {
@@ -224,16 +231,43 @@ pub fn eval_query_value(
         QueryValue::Duration {} => Ok(Some(Value::Integer(
             http_response.duration.as_millis() as i64
         ))),
-        QueryValue::Bytes {} => Ok(Some(Value::Bytes(http_response.body))),
+        QueryValue::Bytes {} => match http_response.uncompress_body() {
+            Ok(s) => Ok(Some(Value::Bytes(s))),
+            Err(inner) => Err(Error {
+                source_info: query.source_info.clone(),
+                inner: RunnerError::from(inner),
+                assert: false,
+            }),
+        },
         QueryValue::Sha256 {} => {
+            let bytes = match http_response.uncompress_body() {
+                Ok(s) => s,
+                Err(inner) => {
+                    return Err(Error {
+                        source_info: query.source_info.clone(),
+                        inner: RunnerError::from(inner),
+                        assert: false,
+                    })
+                }
+            };
             let mut hasher = sha2::Sha256::new();
-            hasher.update(http_response.body);
+            hasher.update(bytes);
             let result = hasher.finalize();
             let bytes = Value::Bytes(result[..].to_vec());
             Ok(Some(bytes))
         }
         QueryValue::Md5 {} => {
-            let bytes = md5::compute(http_response.body).to_vec();
+            let bytes = match http_response.uncompress_body() {
+                Ok(s) => s,
+                Err(inner) => {
+                    return Err(Error {
+                        source_info: query.source_info.clone(),
+                        inner: RunnerError::from(inner),
+                        assert: false,
+                    })
+                }
+            };
+            let bytes = md5::compute(bytes).to_vec();
             Ok(Some(Value::Bytes(bytes)))
         }
     }
@@ -241,7 +275,7 @@ pub fn eval_query_value(
 
 pub fn eval_cookie_attribute_name(
     cookie_attribute_name: CookieAttributeName,
-    cookie: cookie::ResponseCookie,
+    cookie: http::ResponseCookie,
 ) -> Option<Value> {
     match cookie_attribute_name {
         CookieAttributeName::Value(_) => Some(Value::String(cookie.value)),
@@ -305,10 +339,10 @@ pub mod tests {
         // xpath ???
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 13),
+            source_info: SourceInfo::new(1, 1, 1, 13),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -317,7 +351,7 @@ pub mod tests {
                         value: "???".to_string(),
                         encoded: "???".to_string(),
                     }],
-                    source_info: SourceInfo::init(1, 7, 1, 10),
+                    source_info: SourceInfo::new(1, 7, 1, 10),
                 },
             },
             subquery: None,
@@ -327,10 +361,10 @@ pub mod tests {
     pub fn xpath_count_user_query() -> Query {
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 13),
+            source_info: SourceInfo::new(1, 1, 1, 13),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -339,7 +373,7 @@ pub mod tests {
                         value: "count(//user)".to_string(),
                         encoded: "count(//user)".to_string(),
                     }],
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                 },
             },
             subquery: None,
@@ -349,10 +383,10 @@ pub mod tests {
     pub fn xpath_users() -> Query {
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 13),
+            source_info: SourceInfo::new(1, 1, 1, 13),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -361,7 +395,7 @@ pub mod tests {
                         value: "//user".to_string(),
                         encoded: "/user".to_string(),
                     }],
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                 },
             },
             subquery: None,
@@ -392,11 +426,11 @@ pub mod tests {
     pub fn jsonpath_success() -> Query {
         // jsonpath $.success
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 19),
+            source_info: SourceInfo::new(1, 1, 1, 19),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 9, 1, 10),
+                    source_info: SourceInfo::new(1, 9, 1, 10),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -405,7 +439,7 @@ pub mod tests {
                     }],
                     quotes: true,
                     //delimiter: "".to_string(),
-                    source_info: SourceInfo::init(1, 10, 1, 19),
+                    source_info: SourceInfo::new(1, 10, 1, 19),
                 },
             },
             subquery: None,
@@ -415,11 +449,11 @@ pub mod tests {
     pub fn jsonpath_errors() -> Query {
         // jsonpath $.errors
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 19),
+            source_info: SourceInfo::new(1, 1, 1, 19),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 9, 1, 10),
+                    source_info: SourceInfo::new(1, 9, 1, 10),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -428,7 +462,7 @@ pub mod tests {
                     }],
                     quotes: true,
                     //   delimiter: "".to_string(),
-                    source_info: SourceInfo::init(1, 10, 1, 18),
+                    source_info: SourceInfo::new(1, 10, 1, 18),
                 },
             },
             subquery: None,
@@ -438,11 +472,11 @@ pub mod tests {
     pub fn jsonpath_errors_count() -> Query {
         // jsonpath "$.errors" count
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 19),
+            source_info: SourceInfo::new(1, 1, 1, 19),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 9, 1, 10),
+                    source_info: SourceInfo::new(1, 9, 1, 10),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -451,16 +485,16 @@ pub mod tests {
                     }],
                     quotes: true,
                     //   delimiter: "".to_string(),
-                    source_info: SourceInfo::init(1, 10, 1, 18),
+                    source_info: SourceInfo::new(1, 10, 1, 18),
                 },
             },
             subquery: Some((
                 Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                 },
                 Subquery {
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                     value: SubqueryValue::Count {},
                 },
             )),
@@ -470,11 +504,11 @@ pub mod tests {
     pub fn jsonpath_duration() -> Query {
         // jsonpath $.errors
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 19),
+            source_info: SourceInfo::new(1, 1, 1, 19),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 9, 1, 10),
+                    source_info: SourceInfo::new(1, 9, 1, 10),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -483,7 +517,7 @@ pub mod tests {
                     }],
                     quotes: true,
                     //   delimiter: "".to_string(),
-                    source_info: SourceInfo::init(1, 10, 1, 18),
+                    source_info: SourceInfo::new(1, 10, 1, 18),
                 },
             },
             subquery: None,
@@ -493,20 +527,20 @@ pub mod tests {
     pub fn regex_name() -> Query {
         // regex "Hello ([a-zA-Z]+)!"
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 26),
+            source_info: SourceInfo::new(1, 1, 1, 26),
             value: QueryValue::Regex {
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 6, 1, 7),
+                    source_info: SourceInfo::new(1, 6, 1, 7),
                 },
-                expr: Template {
+                value: RegexValue::Template(Template {
                     quotes: true,
                     elements: vec![TemplateElement::String {
                         value: "Hello ([a-zA-Z]+)!".to_string(),
                         encoded: "Hello ([a-zA-Z]+)!".to_string(),
                     }],
-                    source_info: SourceInfo::init(1, 7, 1, 26),
-                },
+                    source_info: SourceInfo::new(1, 7, 1, 26),
+                }),
             },
             subquery: None,
         }
@@ -515,20 +549,20 @@ pub mod tests {
     pub fn regex_invalid() -> Query {
         // regex ???"
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 26),
+            source_info: SourceInfo::new(1, 1, 1, 26),
             value: QueryValue::Regex {
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 6, 1, 7),
+                    source_info: SourceInfo::new(1, 6, 1, 7),
                 },
-                expr: Template {
+                value: RegexValue::Template(Template {
                     quotes: true,
                     elements: vec![TemplateElement::String {
                         value: "???".to_string(),
                         encoded: "???".to_string(),
                     }],
-                    source_info: SourceInfo::init(1, 7, 1, 10),
-                },
+                    source_info: SourceInfo::new(1, 7, 1, 10),
+                }),
             },
             subquery: None,
         }
@@ -539,13 +573,13 @@ pub mod tests {
         let variables = HashMap::new();
         assert_eq!(
             eval_query(
-                Query {
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                &Query {
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                     value: QueryValue::Status {},
                     subquery: None
                 },
                 &variables,
-                http::hello_http_response(),
+                &http::hello_http_response(),
             )
             .unwrap()
             .unwrap(),
@@ -558,11 +592,11 @@ pub mod tests {
         let variables = HashMap::new();
         // header Custom
         let query_header = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Header {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::init(1, 7, 1, 8),
+                    source_info: SourceInfo::new(1, 7, 1, 8),
                 },
                 name: Template {
                     quotes: true,
@@ -570,7 +604,7 @@ pub mod tests {
                         value: "Custom".to_string(),
                         encoded: "Custom".to_string(),
                     }],
-                    source_info: SourceInfo::init(2, 8, 2, 14),
+                    source_info: SourceInfo::new(2, 8, 2, 14),
                 },
             },
             subquery: None,
@@ -579,7 +613,7 @@ pub mod tests {
         //    assert_eq!(error.source_info.start, Pos { line: 1, column: 8 });
         //    assert_eq!(error.inner, RunnerError::QueryHeaderNotFound);
         assert_eq!(
-            eval_query(query_header, &variables, http::hello_http_response()).unwrap(),
+            eval_query(&query_header, &variables, &http::hello_http_response()).unwrap(),
             None
         );
     }
@@ -589,11 +623,11 @@ pub mod tests {
         // header Content-Type
         let variables = HashMap::new();
         let query_header = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Header {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::init(1, 7, 1, 8),
+                    source_info: SourceInfo::new(1, 7, 1, 8),
                 },
                 name: Template {
                     quotes: true,
@@ -601,13 +635,13 @@ pub mod tests {
                         value: "Content-Type".to_string(),
                         encoded: "Content-Type".to_string(),
                     }],
-                    source_info: SourceInfo::init(1, 8, 1, 16),
+                    source_info: SourceInfo::new(1, 8, 1, 16),
                 },
             },
             subquery: None,
         };
         assert_eq!(
-            eval_query(query_header, &variables, http::hello_http_response())
+            eval_query(&query_header, &variables, &http::hello_http_response())
                 .unwrap()
                 .unwrap(),
             Value::String(String::from("text/html; charset=utf-8"))
@@ -619,7 +653,7 @@ pub mod tests {
         let variables = HashMap::new();
         let space = Whitespace {
             value: String::from(""),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         let response = http::Response {
             version: http::Version::Http10,
@@ -636,7 +670,7 @@ pub mod tests {
 
         // cookie "LSID"
         let query = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -646,7 +680,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::init(0, 0, 0, 0),
+                        source_info: SourceInfo::new(0, 0, 0, 0),
                     },
                     attribute: None,
                 },
@@ -654,7 +688,7 @@ pub mod tests {
             subquery: None,
         };
         assert_eq!(
-            eval_query(query, &variables, response.clone())
+            eval_query(&query, &variables, &response.clone())
                 .unwrap()
                 .unwrap(),
             Value::String("DQAAAKEaem_vYg".to_string())
@@ -662,7 +696,7 @@ pub mod tests {
 
         // cookie "LSID[Path]"
         let query = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -672,7 +706,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::init(0, 0, 0, 0),
+                        source_info: SourceInfo::new(0, 0, 0, 0),
                     },
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
@@ -684,7 +718,7 @@ pub mod tests {
             subquery: None,
         };
         assert_eq!(
-            eval_query(query, &variables, response.clone())
+            eval_query(&query, &variables, &response.clone())
                 .unwrap()
                 .unwrap(),
             Value::String("/accounts".to_string())
@@ -692,7 +726,7 @@ pub mod tests {
 
         // cookie "LSID[Secure]"
         let query = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -702,7 +736,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::init(0, 0, 0, 0),
+                        source_info: SourceInfo::new(0, 0, 0, 0),
                     },
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
@@ -714,7 +748,7 @@ pub mod tests {
             subquery: None,
         };
         assert_eq!(
-            eval_query(query, &variables, response.clone())
+            eval_query(&query, &variables, &response.clone())
                 .unwrap()
                 .unwrap(),
             Value::Unit
@@ -722,7 +756,7 @@ pub mod tests {
 
         // cookie "LSID[Domain]"
         let query = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -732,7 +766,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::init(0, 0, 0, 0),
+                        source_info: SourceInfo::new(0, 0, 0, 0),
                     },
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
@@ -743,28 +777,28 @@ pub mod tests {
             },
             subquery: None,
         };
-        assert_eq!(eval_query(query, &variables, response).unwrap(), None);
+        assert_eq!(eval_query(&query, &variables, &response).unwrap(), None);
     }
 
     #[test]
     fn test_eval_cookie_attribute_name() {
-        let cookie = cookie::ResponseCookie {
+        let cookie = http::ResponseCookie {
             name: "LSID".to_string(),
             value: "DQAAAKEaem_vYg".to_string(),
             attributes: vec![
-                cookie::CookieAttribute {
+                http::CookieAttribute {
                     name: "Path".to_string(),
                     value: Some("/accounts".to_string()),
                 },
-                cookie::CookieAttribute {
+                http::CookieAttribute {
                     name: "Expires".to_string(),
                     value: Some("Wed, 13 Jan 2021 22:23:01 GMT".to_string()),
                 },
-                cookie::CookieAttribute {
+                http::CookieAttribute {
                     name: "Secure".to_string(),
                     value: None,
                 },
-                cookie::CookieAttribute {
+                http::CookieAttribute {
                     name: "HttpOnly".to_string(),
                     value: None,
                 },
@@ -829,30 +863,30 @@ pub mod tests {
         let variables = HashMap::new();
         assert_eq!(
             eval_query(
-                Query {
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                &Query {
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                     value: QueryValue::Body {},
                     subquery: None
                 },
                 &variables,
-                http::hello_http_response(),
+                &http::hello_http_response(),
             )
             .unwrap()
             .unwrap(),
             Value::String(String::from("Hello World!"))
         );
         let error = eval_query(
-            Query {
-                source_info: SourceInfo::init(1, 1, 1, 2),
+            &Query {
+                source_info: SourceInfo::new(1, 1, 1, 2),
                 value: QueryValue::Body {},
                 subquery: None,
             },
             &variables,
-            http::bytes_http_response(),
+            &http::bytes_http_response(),
         )
         .err()
         .unwrap();
-        assert_eq!(error.source_info, SourceInfo::init(1, 1, 1, 2));
+        assert_eq!(error.source_info, SourceInfo::new(1, 1, 1, 2));
         assert_eq!(
             error.inner,
             RunnerError::InvalidDecoding {
@@ -871,7 +905,7 @@ pub mod tests {
             body: vec![200],
             duration: Default::default(),
         };
-        let error = eval_query(xpath_users(), &variables, http_response)
+        let error = eval_query(&xpath_users(), &variables, &http_response)
             .err()
             .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 1 });
@@ -888,11 +922,11 @@ pub mod tests {
         let variables = HashMap::new();
         // xpath ^^^
         let query = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Xpath {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::init(1, 6, 1, 7),
+                    source_info: SourceInfo::new(1, 6, 1, 7),
                 },
                 expr: Template {
                     quotes: true,
@@ -900,12 +934,12 @@ pub mod tests {
                         value: "^^^".to_string(),
                         encoded: "^^^".to_string(),
                     }],
-                    source_info: SourceInfo::init(1, 7, 1, 10),
+                    source_info: SourceInfo::new(1, 7, 1, 10),
                 },
             },
             subquery: None,
         };
-        let error = eval_query(query, &variables, http::xml_two_users_http_response())
+        let error = eval_query(&query, &variables, &http::xml_two_users_http_response())
             .err()
             .unwrap();
         assert_eq!(error.inner, RunnerError::QueryInvalidXpathEval);
@@ -918,9 +952,9 @@ pub mod tests {
 
         assert_eq!(
             eval_query(
-                xpath_users(),
+                &xpath_users(),
                 &variables,
-                http::xml_two_users_http_response(),
+                &http::xml_two_users_http_response(),
             )
             .unwrap()
             .unwrap(),
@@ -928,9 +962,9 @@ pub mod tests {
         );
         assert_eq!(
             eval_query(
-                xpath_count_user_query(),
+                &xpath_count_user_query(),
                 &variables,
-                http::xml_two_users_http_response(),
+                &http::xml_two_users_http_response(),
             )
             .unwrap()
             .unwrap(),
@@ -943,10 +977,10 @@ pub mod tests {
         // $x("normalize-space(/html/head/meta/@charset)")
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         };
         Query {
-            source_info: SourceInfo::init(1, 1, 1, 13),
+            source_info: SourceInfo::new(1, 1, 1, 13),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -955,7 +989,7 @@ pub mod tests {
                         value: "normalize-space(/html/head/meta/@charset)".to_string(),
                         encoded: "normalize-space(/html/head/meta/@charset)".to_string(),
                     }],
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                 },
             },
             subquery: None,
@@ -966,9 +1000,13 @@ pub mod tests {
     fn test_query_xpath_with_html() {
         let variables = HashMap::new();
         assert_eq!(
-            eval_query(xpath_html_charset(), &variables, http::html_http_response())
-                .unwrap()
-                .unwrap(),
+            eval_query(
+                &xpath_html_charset(),
+                &variables,
+                &http::html_http_response()
+            )
+            .unwrap()
+            .unwrap(),
             Value::String(String::from("UTF-8"))
         );
     }
@@ -979,11 +1017,11 @@ pub mod tests {
 
         // jsonpath xxx
         let jsonpath_query = Query {
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 9, 1, 10),
+                    source_info: SourceInfo::new(1, 9, 1, 10),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -992,13 +1030,13 @@ pub mod tests {
                     }],
                     quotes: true,
                     // delimiter: "".to_string(),
-                    source_info: SourceInfo::init(1, 10, 1, 13),
+                    source_info: SourceInfo::new(1, 10, 1, 13),
                 },
             },
             subquery: None,
         };
 
-        let error = eval_query(jsonpath_query, &variables, json_http_response())
+        let error = eval_query(&jsonpath_query, &variables, &json_http_response())
             .err()
             .unwrap();
         assert_eq!(
@@ -1026,7 +1064,7 @@ pub mod tests {
             body: String::into_bytes(String::from("xxx")),
             duration: Default::default(),
         };
-        let error = eval_query(jsonpath_success(), &variables, http_response)
+        let error = eval_query(&jsonpath_success(), &variables, &http_response)
             .err()
             .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 1 });
@@ -1045,7 +1083,7 @@ pub mod tests {
         };
         //assert_eq!(jsonpath_success().eval(http_response).unwrap(), Value::List(vec![]));
         assert_eq!(
-            eval_query(jsonpath_success(), &variables, http_response).unwrap(),
+            eval_query(&jsonpath_success(), &variables, &http_response).unwrap(),
             None
         );
     }
@@ -1054,13 +1092,13 @@ pub mod tests {
     fn test_query_json() {
         let variables = HashMap::new();
         assert_eq!(
-            eval_query(jsonpath_success(), &variables, json_http_response())
+            eval_query(&jsonpath_success(), &variables, &json_http_response())
                 .unwrap()
                 .unwrap(),
             Value::Bool(false)
         );
         assert_eq!(
-            eval_query(jsonpath_errors(), &variables, json_http_response())
+            eval_query(&jsonpath_errors(), &variables, &json_http_response())
                 .unwrap()
                 .unwrap(),
             Value::List(vec![
@@ -1076,7 +1114,7 @@ pub mod tests {
         );
 
         assert_eq!(
-            eval_query(jsonpath_errors_count(), &variables, json_http_response())
+            eval_query(&jsonpath_errors_count(), &variables, &json_http_response())
                 .unwrap()
                 .unwrap(),
             Value::Integer(2)
@@ -1087,16 +1125,16 @@ pub mod tests {
     fn test_query_regex() {
         let variables = HashMap::new();
         assert_eq!(
-            eval_query(regex_name(), &variables, http::hello_http_response())
+            eval_query(&regex_name(), &variables, &http::hello_http_response())
                 .unwrap()
                 .unwrap(),
             Value::String("World".to_string())
         );
 
-        let error = eval_query(regex_invalid(), &variables, http::hello_http_response())
+        let error = eval_query(&regex_invalid(), &variables, &http::hello_http_response())
             .err()
             .unwrap();
-        assert_eq!(error.source_info, SourceInfo::init(1, 7, 1, 10));
+        assert_eq!(error.source_info, SourceInfo::new(1, 7, 1, 10));
         assert_eq!(error.inner, RunnerError::InvalidRegex());
     }
 
@@ -1105,13 +1143,13 @@ pub mod tests {
         let variables = HashMap::new();
         assert_eq!(
             eval_query(
-                Query {
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                &Query {
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                     value: QueryValue::Bytes {},
                     subquery: None
                 },
                 &variables,
-                http::hello_http_response(),
+                &http::hello_http_response(),
             )
             .unwrap()
             .unwrap(),
@@ -1124,13 +1162,13 @@ pub mod tests {
         let variables = HashMap::new();
         assert_eq!(
             eval_query(
-                Query {
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                &Query {
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                     value: QueryValue::Sha256 {},
                     subquery: None
                 },
                 &variables,
-                http::Response {
+                &http::Response {
                     version: http::Version::Http10,
                     status: 200,
                     headers: vec![],
