@@ -44,20 +44,20 @@ fn eval_template_element(
 ) -> Result<String, Error> {
     match template_element {
         TemplateElement::String { value, .. } => Ok(value.clone()),
-        TemplateElement::Expression(expr) => eval_expression(expr.clone(), variables),
+        TemplateElement::Expression(expr) => eval_expression(expr, variables),
     }
 }
 
-pub fn eval_expression(expr: Expr, variables: &HashMap<String, Value>) -> Result<String, Error> {
-    let source_info = expr.variable.source_info;
-    let name = expr.variable.name;
+pub fn eval_expression(expr: &Expr, variables: &HashMap<String, Value>) -> Result<String, Error> {
+    let source_info = &expr.variable.source_info;
+    let name = &expr.variable.name;
     match variables.get(name.as_str()) {
         Some(value) => {
             if value.is_renderable() {
                 Ok(value.clone().to_string())
             } else {
                 Err(Error {
-                    source_info,
+                    source_info: source_info.clone(),
                     inner: RunnerError::UnrenderableVariable {
                         value: value.to_string(),
                     },
@@ -66,8 +66,8 @@ pub fn eval_expression(expr: Expr, variables: &HashMap<String, Value>) -> Result
             }
         }
         _ => Err(Error {
-            source_info,
-            inner: RunnerError::TemplateVariableNotDefined { name },
+            source_info: source_info.clone(),
+            inner: RunnerError::TemplateVariableNotDefined { name: name.clone() },
             assert: false,
         }),
     }
@@ -93,15 +93,15 @@ mod tests {
         TemplateElement::Expression(Expr {
             space0: Whitespace {
                 value: "".to_string(),
-                source_info: SourceInfo::init(1, 3, 1, 3),
+                source_info: SourceInfo::new(1, 3, 1, 3),
             },
             variable: Variable {
                 name: "name".to_string(),
-                source_info: SourceInfo::init(1, 3, 1, 7),
+                source_info: SourceInfo::new(1, 3, 1, 7),
             },
             space1: Whitespace {
                 value: "".to_string(),
-                source_info: SourceInfo::init(1, 7, 1, 7),
+                source_info: SourceInfo::new(1, 7, 1, 7),
             },
         })
     }
@@ -139,7 +139,7 @@ mod tests {
         let error = eval_template_element(&template_element_expression(), &variables)
             .err()
             .unwrap();
-        assert_eq!(error.source_info, SourceInfo::init(1, 3, 1, 7));
+        assert_eq!(error.source_info, SourceInfo::new(1, 3, 1, 7));
         assert_eq!(
             error.inner,
             RunnerError::UnrenderableVariable {
