@@ -1,5 +1,23 @@
+/*
+ * Hurl (https://hurl.dev)
+ * Copyright (C) 2022 Orange
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *          http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ *
+ */
+
+use hurl::cli::Logger;
 use std::default::Default;
-use std::path::Path;
 use std::time::Duration;
 
 use hurl::http::*;
@@ -11,15 +29,10 @@ pub fn new_header(name: &str, value: &str) -> Header {
     }
 }
 
-fn default_client() -> Client {
-    let options = ClientOptions::default();
-    Client::init(options)
-}
-
-fn default_get_request(url: String) -> RequestSpec {
+fn default_get_request(url: &str) -> RequestSpec {
     RequestSpec {
         method: Method::Get,
-        url,
+        url: url.to_string(),
         headers: vec![],
         querystring: vec![],
         form: vec![],
@@ -34,14 +47,17 @@ fn default_get_request(url: String) -> RequestSpec {
 
 #[test]
 fn test_hello() {
-    let mut client = default_client();
-    let request_spec = default_get_request("http://localhost:8000/hello".to_string());
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("http://localhost:8000/hello");
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/hello'".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "GET".to_string());
     assert_eq!(request.url, "http://localhost:8000/hello".to_string());
     assert_eq!(request.headers.len(), 3);
@@ -67,7 +83,7 @@ fn test_hello() {
         name: "Content-Type".to_string(),
         value: "text/html; charset=utf-8".to_string(),
     }));
-    assert_eq!(response.get_header_values("Date".to_string()).len(), 1);
+    assert_eq!(response.get_header_values("Date").len(), 1);
 }
 
 // endregion
@@ -76,7 +92,10 @@ fn test_hello() {
 
 #[test]
 fn test_put() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Put,
         url: "http://localhost:8000/put".to_string(),
@@ -89,11 +108,11 @@ fn test_put() {
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/put' -X PUT".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "PUT".to_string());
     assert_eq!(request.url, "http://localhost:8000/put".to_string());
     assert!(request.headers.contains(&Header {
@@ -111,7 +130,10 @@ fn test_put() {
 
 #[test]
 fn test_patch() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Patch,
         url: "http://localhost:8000/patch/file.txt".to_string(),
@@ -137,11 +159,11 @@ fn test_patch() {
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/patch/file.txt' -X PATCH -H 'Host: www.example.com' -H 'Content-Type: application/example' -H 'If-Match: \"e0023aa4e\"'".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "PATCH".to_string());
     assert_eq!(
         request.url,
@@ -166,7 +188,10 @@ fn test_patch() {
 
 #[test]
 fn test_custom_headers() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Get,
         url: "http://localhost:8000/custom-headers".to_string(),
@@ -184,13 +209,13 @@ fn test_custom_headers() {
         body: Body::Binary(vec![]),
         content_type: None,
     };
-    assert!(client.options.curl_args().is_empty());
+    assert!(options.curl_args().is_empty());
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/custom-headers' -H 'Fruit: Raspberry' -H 'Fruit: Apple' -H 'Fruit: Banana' -H 'Fruit: Grape' -H 'Color: Green'".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "GET".to_string());
     assert_eq!(
         request.url,
@@ -210,7 +235,10 @@ fn test_custom_headers() {
 
 #[test]
 fn test_querystring_params() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Get,
         url: "http://localhost:8000/querystring-params".to_string(),
@@ -240,10 +268,10 @@ fn test_querystring_params() {
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/querystring-params?param1=value1&param2=&param3=a%3Db&param4=1%2C2%2C3'".to_string()
     );
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "GET".to_string());
     assert_eq!(request.url, "http://localhost:8000/querystring-params?param1=value1&param2=&param3=a%3Db&param4=1%2C2%2C3".to_string());
     assert_eq!(request.headers.len(), 3);
@@ -258,7 +286,10 @@ fn test_querystring_params() {
 
 #[test]
 fn test_form_params() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Post,
         url: "http://localhost:8000/form-params".to_string(),
@@ -296,11 +327,11 @@ fn test_form_params() {
         content_type: Some("application/x-www-form-urlencoded".to_string()),
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/form-params' --data 'param1=value1' --data 'param2=' --data 'param3=a%3Db' --data 'param4=a%253db' --data 'values[0]=0' --data 'values[1]=1'".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "POST".to_string());
     assert_eq!(request.url, "http://localhost:8000/form-params".to_string());
     assert!(request.headers.contains(&Header {
@@ -312,8 +343,8 @@ fn test_form_params() {
     assert!(response.body.is_empty());
 
     // make sure you can reuse client for other request
-    let request = default_get_request("http://localhost:8000/hello".to_string());
-    let (request, response) = client.execute(&request).unwrap();
+    let request = default_get_request("http://localhost:8000/hello");
+    let (request, response) = client.execute(&request, &options, &logger).unwrap();
     assert_eq!(request.method, "GET".to_string());
     assert_eq!(request.url, "http://localhost:8000/hello".to_string());
     assert_eq!(request.headers.len(), 3);
@@ -327,20 +358,18 @@ fn test_form_params() {
 
 #[test]
 fn test_redirect() {
-    let request_spec = default_get_request("http://localhost:8000/redirect".to_string());
-
-    let mut client = default_client();
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let request_spec = default_get_request("http://localhost:8000/redirect");
+    let logger = Logger::new(false, false, "", "");
+    let options = ClientOptions::default();
+    let mut client = Client::new(None);
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "GET".to_string());
     assert_eq!(request.url, "http://localhost:8000/redirect".to_string());
     assert_eq!(request.headers.len(), 3);
 
     assert_eq!(response.status, 302);
     assert_eq!(
-        response
-            .get_header_values("Location".to_string())
-            .get(0)
-            .unwrap(),
+        response.get_header_values("Location").get(0).unwrap(),
         "http://localhost:8000/redirected"
     );
     assert_eq!(client.redirect_count, 0);
@@ -348,20 +377,23 @@ fn test_redirect() {
 
 #[test]
 fn test_follow_location() {
-    let request_spec = default_get_request("http://localhost:8000/redirect".to_string());
-
+    let request_spec = default_get_request("http://localhost:8000/redirect");
+    let logger = Logger::new(false, false, "", "");
     let options = ClientOptions {
         follow_location: true,
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    assert_eq!(client.options.curl_args(), vec!["-L".to_string(),]);
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    assert_eq!(options.curl_args(), vec!["-L".to_string()]);
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/redirect' -L".to_string()
     );
 
-    let calls = client.execute_with_redirect(&request_spec).unwrap();
+    let calls = client
+        .execute_with_redirect(&request_spec, &options, &logger)
+        .unwrap();
     assert_eq!(calls.len(), 2);
 
     let (request1, response1) = calls.get(0).unwrap();
@@ -383,8 +415,10 @@ fn test_follow_location() {
     assert_eq!(client.redirect_count, 1);
 
     // make sure that the redirect count is reset to 0
-    let request = default_get_request("http://localhost:8000/hello".to_string());
-    let calls = client.execute_with_redirect(&request).unwrap();
+    let request = default_get_request("http://localhost:8000/hello");
+    let calls = client
+        .execute_with_redirect(&request, &options, &logger)
+        .unwrap();
     let (_, response) = calls.get(0).unwrap();
     assert_eq!(response.status, 200);
     assert_eq!(response.body, b"Hello World!".to_vec());
@@ -398,22 +432,29 @@ fn test_max_redirect() {
         max_redirect: Some(10),
         ..Default::default()
     };
-    let mut client = Client::init(options);
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
 
-    let request_spec = default_get_request("http://localhost:8000/redirect/15".to_string());
+    let request_spec = default_get_request("http://localhost:8000/redirect/15");
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/redirect/15' -L --max-redirs 10".to_string()
     );
-    let error = client.execute_with_redirect(&request_spec).err().unwrap();
+    let error = client
+        .execute_with_redirect(&request_spec, &options, &logger)
+        .err()
+        .unwrap();
     assert_eq!(error, HttpError::TooManyRedirect);
 
-    let request_spec = default_get_request("http://localhost:8000/redirect/8".to_string());
+    let request_spec = default_get_request("http://localhost:8000/redirect/8");
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/redirect/8' -L --max-redirs 10".to_string()
     );
-    let calls = client.execute_with_redirect(&request_spec).unwrap();
+    let calls = client
+        .execute_with_redirect(&request_spec, &options, &logger)
+        .unwrap();
     let (request, response) = calls.last().unwrap();
     assert_eq!(request.url, "http://localhost:8000/redirect/0".to_string());
     assert_eq!(response.status, 200);
@@ -426,7 +467,10 @@ fn test_max_redirect() {
 
 #[test]
 fn test_multipart_form_data() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Post,
         url: "http://localhost:8000/multipart-form-data".to_string(),
@@ -462,11 +506,11 @@ fn test_multipart_form_data() {
         content_type: Some("multipart/form-data".to_string()),
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/multipart-form-data' -F 'key1=value1' -F 'upload1=@data.txt;type=text/plain' -F 'upload2=@data.html;type=text/html' -F 'upload3=@data.txt;type=text/html'".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Content-Length".to_string(),
         value: "627".to_string(),
@@ -475,8 +519,8 @@ fn test_multipart_form_data() {
     assert!(response.body.is_empty());
 
     // make sure you can reuse client for other request
-    let request_spec = default_get_request("http://localhost:8000/hello".to_string());
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let request_spec = default_get_request("http://localhost:8000/hello");
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.method, "GET".to_string());
     assert_eq!(response.status, 200);
     assert_eq!(response.body, b"Hello World!".to_vec());
@@ -488,7 +532,10 @@ fn test_multipart_form_data() {
 
 #[test]
 fn test_post_bytes() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Post,
         url: "http://localhost:8000/post-base64".to_string(),
@@ -501,10 +548,10 @@ fn test_post_bytes() {
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/post-base64' -H 'Content-Type: application/octet-stream' --data $'\\x48\\x65\\x6c\\x6c\\x6f\\x20\\x57\\x6f\\x72\\x6c\\x64\\x21'".to_string()
     );
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Content-Length".to_string(),
         value: "12".to_string(),
@@ -518,7 +565,10 @@ fn test_post_bytes() {
 
 #[test]
 fn test_expect() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Post,
         url: "http://localhost:8000/expect".to_string(),
@@ -534,11 +584,11 @@ fn test_expect() {
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/expect' -H 'Expect: 100-continue' -H 'Content-Type:' --data 'data'".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Expect".to_string(),
         value: "100-continue".to_string(),
@@ -551,10 +601,12 @@ fn test_expect() {
 #[test]
 fn test_basic_authentication() {
     let options = ClientOptions {
-        user: Some("bob:secret".to_string()),
+        user: Some("bob@email.com:secret".to_string()),
         ..Default::default()
     };
-    let mut client = Client::init(options);
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Get,
         url: "http://localhost:8000/basic-authentication".to_string(),
@@ -567,22 +619,24 @@ fn test_basic_authentication() {
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
-        "curl 'http://localhost:8000/basic-authentication' --user 'bob:secret'".to_string()
+        client.curl_command_line(&request_spec, &context_dir, &options),
+        "curl 'http://localhost:8000/basic-authentication' --user 'bob@email.com:secret'"
+            .to_string()
     );
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Authorization".to_string(),
-        value: "Basic Ym9iOnNlY3JldA==".to_string(),
+        value: "Basic Ym9iQGVtYWlsLmNvbTpzZWNyZXQ=".to_string(),
     }));
     assert_eq!(response.status, 200);
     assert_eq!(response.version, Version::Http10);
     assert_eq!(response.body, b"You are authenticated".to_vec());
 
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let mut client = Client::new(None);
     let request_spec = RequestSpec {
         method: Method::Get,
-        url: "http://bob:secret@localhost:8000/basic-authentication".to_string(),
+        url: "http://bob%40email.com:secret@localhost:8000/basic-authentication".to_string(),
         headers: vec![],
         querystring: vec![],
         form: vec![],
@@ -592,13 +646,13 @@ fn test_basic_authentication() {
         content_type: None,
     };
     assert_eq!(
-        request_spec.curl_args(Path::new("")),
-        vec!["'http://bob:secret@localhost:8000/basic-authentication'".to_string()]
+        request_spec.curl_args(&ContextDir::default()),
+        vec!["'http://bob%40email.com:secret@localhost:8000/basic-authentication'".to_string()]
     );
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Authorization".to_string(),
-        value: "Basic Ym9iOnNlY3JldA==".to_string(),
+        value: "Basic Ym9iQGVtYWlsLmNvbTpzZWNyZXQ=".to_string(),
     }));
     assert_eq!(response.status, 200);
     assert_eq!(response.version, Version::Http10);
@@ -611,9 +665,10 @@ fn test_cacert() {
         cacert_file: Some("tests/cert.pem".to_string()),
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    let request_spec = default_get_request("https://localhost:8001/hello".to_string());
-    let (_, response) = client.execute(&request_spec).unwrap();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("https://localhost:8001/hello");
+    let (_, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(response.status, 200);
 }
 
@@ -621,39 +676,67 @@ fn test_cacert() {
 
 #[test]
 fn test_error_could_not_resolve_host() {
-    let mut client = default_client();
-    let request = default_get_request("http://unknown".to_string());
-    let error = client.execute(&request).err().unwrap();
+    let options = ClientOptions::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("http://unknown");
+    let error = client
+        .execute(&request_spec, &options, &logger)
+        .err()
+        .unwrap();
     assert!(matches!(error, HttpError::Libcurl { .. }));
-    if let HttpError::Libcurl { code, description } = error {
+    if let HttpError::Libcurl {
+        code,
+        description,
+        url,
+    } = error
+    {
         assert_eq!(code, 6);
         assert_eq!(description, "Could not resolve host: unknown");
+        assert_eq!(url, "http://unknown");
     }
 }
 
 #[test]
 fn test_error_fail_to_connect() {
-    let mut client = default_client();
-    let request_spec = default_get_request("http://localhost:9999".to_string());
-    let error = client.execute(&request_spec).err().unwrap();
+    let options = ClientOptions::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("http://localhost:9999");
+    let error = client
+        .execute(&request_spec, &options, &logger)
+        .err()
+        .unwrap();
     assert!(matches!(error, HttpError::Libcurl { .. }));
-    if let HttpError::Libcurl { code, description } = error {
+    if let HttpError::Libcurl {
+        code,
+        description,
+        url,
+    } = error
+    {
         assert_eq!(code, 7);
         assert!(description.starts_with("Failed to connect to localhost port 9999"));
+        assert_eq!(url, "http://localhost:9999");
     }
 
     let options = ClientOptions {
         proxy: Some("localhost:9999".to_string()),
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    let request = default_get_request("http://localhost:8000/hello".to_string());
-    let error = client.execute(&request).err().unwrap();
+    let mut client = Client::new(None);
+    let request = default_get_request("http://localhost:8000/hello");
+    let error = client.execute(&request, &options, &logger).err().unwrap();
     assert!(matches!(error, HttpError::Libcurl { .. }));
-    if let HttpError::Libcurl { code, description } = error {
+    if let HttpError::Libcurl {
+        code,
+        description,
+        url,
+    } = error
+    {
         assert_eq!(code, 7);
         eprintln!("description={}", description);
         assert!(description.starts_with("Failed to connect to localhost port 9999"));
+        assert_eq!(url, "http://localhost:8000/hello");
     }
 }
 
@@ -663,30 +746,53 @@ fn test_error_could_not_resolve_proxy_name() {
         proxy: Some("unknown".to_string()),
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    let request_spec = default_get_request("http://localhost:8000/hello".to_string());
-    let error = client.execute(&request_spec).err().unwrap();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("http://localhost:8000/hello");
+    let error = client
+        .execute(&request_spec, &options, &logger)
+        .err()
+        .unwrap();
     assert!(matches!(error, HttpError::Libcurl { .. }));
-    if let HttpError::Libcurl { code, description } = error {
+    if let HttpError::Libcurl {
+        code,
+        description,
+        url,
+    } = error
+    {
         assert_eq!(code, 5);
         assert_eq!(description, "Could not resolve proxy: unknown");
+        assert_eq!(url, "http://localhost:8000/hello");
     }
 }
 
 #[test]
 fn test_error_ssl() {
     let options = ClientOptions::default();
-    let mut client = Client::init(options);
-    let request_spec = default_get_request("https://localhost:8001/hello".to_string());
-    let error = client.execute(&request_spec).err().unwrap();
-    if let HttpError::Libcurl { code, description } = error {
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("https://localhost:8001/hello");
+    let error = client
+        .execute(&request_spec, &options, &logger)
+        .err()
+        .unwrap();
+    if let HttpError::Libcurl {
+        code,
+        description,
+        url,
+    } = error
+    {
         assert_eq!(code, 60);
-        let message = if cfg!(windows) {
-            "schannel: SEC_E_UNTRUSTED_ROOT (0x80090325) - The certificate chain was issued by an authority that is not trusted.".to_string()
-        } else {
-            "SSL certificate problem: self signed certificate".to_string()
-        };
-        assert_eq!(description, message);
+        let descriptions = [
+            // Windows messages:
+            "schannel: SEC_E_UNTRUSTED_ROOT (0x80090325) - The certificate chain was issued by an authority that is not trusted.".to_string(),
+            // Unix-like, before OpenSSL 3.0.0
+            "SSL certificate problem: self signed certificate".to_string(),
+            // Unix-like, after OpenSSL 3.0.0
+            "SSL certificate problem: self-signed certificate".to_string(),
+        ];
+        assert!(descriptions.contains(&description));
+        assert_eq!(url, "https://localhost:8001/hello");
     }
 }
 
@@ -696,13 +802,23 @@ fn test_timeout() {
         timeout: Duration::from_millis(100),
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    let request_spec = default_get_request("http://localhost:8000/timeout".to_string());
-    let error = client.execute(&request_spec).err().unwrap();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("http://localhost:8000/timeout");
+    let error = client
+        .execute(&request_spec, &options, &logger)
+        .err()
+        .unwrap();
     assert!(matches!(error, HttpError::Libcurl { .. }));
-    if let HttpError::Libcurl { code, description } = error {
+    if let HttpError::Libcurl {
+        code,
+        description,
+        url,
+    } = error
+    {
         assert_eq!(code, 28);
         assert!(description.starts_with("Operation timed out after "));
+        assert_eq!(url, "http://localhost:8000/timeout");
     }
 }
 
@@ -712,7 +828,8 @@ fn test_accept_encoding() {
         compressed: true,
         ..Default::default()
     };
-    let mut client = Client::init(options);
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
 
     let request_spec = RequestSpec {
         method: Method::Get,
@@ -725,7 +842,7 @@ fn test_accept_encoding() {
         body: Body::Binary(vec![]),
         content_type: None,
     };
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Accept-Encoding".to_string(),
         value: "gzip, deflate, br".to_string(),
@@ -743,26 +860,39 @@ fn test_connect_timeout() {
         connect_timeout: Duration::from_secs(1),
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    let request_spec = default_get_request("http://10.0.0.0".to_string());
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("http://10.0.0.0");
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://10.0.0.0' --connect-timeout 1".to_string()
     );
-    let error = client.execute(&request_spec).err().unwrap();
+    let error = client
+        .execute(&request_spec, &options, &logger)
+        .err()
+        .unwrap();
     assert!(matches!(error, HttpError::Libcurl { .. }));
-    if let HttpError::Libcurl { code, description } = error {
+    if let HttpError::Libcurl {
+        code,
+        description,
+        url,
+    } = error
+    {
         eprintln!("description={}", description);
-        if cfg!(target_os = "macos") {
-            assert_eq!(code, 7);
-            assert_eq!(description, "Couldn't connect to server");
-        } else {
-            assert_eq!(code, 28);
-            assert!(
-                description.starts_with("Connection timed out")
-                    || description.starts_with("Connection timeout")
-            );
-        }
+        // TODO: remove the 7 / "Couldn't connect to server" case
+        // On Linux/Windows libcurl version, the correct error message
+        // is 28 / "Connection timeout" | "Connection timed out"
+        // On macOS <= 11.6.4, the built-in libcurl use
+        // 7 / "Couldn't connect to server" errors. On the GitHub CI, macOS images are 11.6.4.
+        // So we keep this code until a newer macOS image is used in the GitHub actions.
+        assert!(code == 7 || code == 28);
+        assert!(
+            description.starts_with("Couldn't connect to server")
+                || description.starts_with("Connection timed out")
+                || description.starts_with("Connection timeout")
+        );
+        assert_eq!(url, "http://10.0.0.0");
     }
 }
 // endregion
@@ -771,7 +901,10 @@ fn test_connect_timeout() {
 
 #[test]
 fn test_cookie() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Get,
         url: "http://localhost:8000/cookies/set-request-cookie1-valueA".to_string(),
@@ -787,14 +920,14 @@ fn test_cookie() {
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/cookies/set-request-cookie1-valueA' --cookie 'cookie1=valueA'"
             .to_string()
     );
 
     //assert_eq!(request.cookies(), vec!["cookie1=valueA".to_string(),]);
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Cookie".to_string(),
         value: "cookie1=valueA".to_string(),
@@ -813,13 +946,16 @@ fn test_cookie() {
         body: Body::Binary(vec![]),
         content_type: None,
     };
-    let (_request, response) = client.execute(&request_spec).unwrap();
+    let (_request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(response.status, 200);
 }
 
 #[test]
 fn test_multiple_request_cookies() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec = RequestSpec {
         method: Method::Get,
         url: "http://localhost:8000/cookies/set-multiple-request-cookies".to_string(),
@@ -836,19 +972,23 @@ fn test_multiple_request_cookies() {
                 name: "user2".to_string(),
                 value: "Bill".to_string(),
             },
+            RequestCookie {
+                name: "user3".to_string(),
+                value: "Bruce".to_string(),
+            },
         ],
         body: Body::Binary(vec![]),
         content_type: None,
     };
     assert_eq!(
-        client.curl_command_line(&request_spec),
-        "curl 'http://localhost:8000/cookies/set-multiple-request-cookies' --cookie 'user1=Bob; user2=Bill'".to_string()
+        client.curl_command_line(&request_spec, &context_dir, &options),
+        "curl 'http://localhost:8000/cookies/set-multiple-request-cookies' --cookie 'user1=Bob; user2=Bill; user3=Bruce'".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Cookie".to_string(),
-        value: "user1=Bob; user2=Bill".to_string(),
+        value: "user1=Bob; user2=Bill; user3=Bruce".to_string(),
     }));
     assert_eq!(response.status, 200);
     assert!(response.body.is_empty());
@@ -856,10 +996,12 @@ fn test_multiple_request_cookies() {
 
 #[test]
 fn test_cookie_storage() {
-    let mut client = default_client();
+    let options = ClientOptions::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
     let request_spec =
-        default_get_request("http://localhost:8000/cookies/set-session-cookie2-valueA".to_string());
-    let (request, response) = client.execute(&request_spec).unwrap();
+        default_get_request("http://localhost:8000/cookies/set-session-cookie2-valueA");
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(
         request.url,
         "http://localhost:8000/cookies/set-session-cookie2-valueA".to_string()
@@ -882,10 +1024,9 @@ fn test_cookie_storage() {
         }
     );
 
-    let request_spec = default_get_request(
-        "http://localhost:8000/cookies/assert-that-cookie2-is-valueA".to_string(),
-    );
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let request_spec =
+        default_get_request("http://localhost:8000/cookies/assert-that-cookie2-is-valueA");
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert!(request.headers.contains(&Header {
         name: "Cookie".to_string(),
         value: "cookie2=valueA".to_string(),
@@ -900,16 +1041,17 @@ fn test_cookie_file() {
         cookie_input_file: Some("tests/cookies.txt".to_string()),
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    let request_spec = default_get_request(
-        "http://localhost:8000/cookies/assert-that-cookie2-is-valueA".to_string(),
-    );
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(Some("tests/cookies.txt".to_string()));
+    let logger = Logger::new(false, false, "", "");
+    let request_spec =
+        default_get_request("http://localhost:8000/cookies/assert-that-cookie2-is-valueA");
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/cookies/assert-that-cookie2-is-valueA' --cookie tests/cookies.txt".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(
         request.url,
         "http://localhost:8000/cookies/assert-that-cookie2-is-valueA"
@@ -934,13 +1076,15 @@ fn test_proxy() {
         proxy: Some("localhost:8888".to_string()),
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    let request_spec = default_get_request("http://localhost:8000/proxy".to_string());
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    let request_spec = default_get_request("http://localhost:8000/proxy");
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'http://localhost:8000/proxy' --proxy 'localhost:8888'".to_string()
     );
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.url, "http://localhost:8000/proxy");
     assert_eq!(response.status, 200);
 }
@@ -953,15 +1097,17 @@ fn test_insecure() {
         insecure: true,
         ..Default::default()
     };
-    let mut client = Client::init(options);
-    assert_eq!(client.options.curl_args(), vec!["--insecure".to_string()]);
-    let request_spec = default_get_request("https://localhost:8001/hello".to_string());
+    let context_dir = ContextDir::default();
+    let mut client = Client::new(None);
+    let logger = Logger::new(false, false, "", "");
+    assert_eq!(options.curl_args(), vec!["--insecure".to_string()]);
+    let request_spec = default_get_request("https://localhost:8001/hello");
     assert_eq!(
-        client.curl_command_line(&request_spec),
+        client.curl_command_line(&request_spec, &context_dir, &options),
         "curl 'https://localhost:8001/hello' --insecure".to_string()
     );
 
-    let (request, response) = client.execute(&request_spec).unwrap();
+    let (request, response) = client.execute(&request_spec, &options, &logger).unwrap();
     assert_eq!(request.url, "https://localhost:8001/hello");
     assert_eq!(response.status, 200);
 }
@@ -981,3 +1127,19 @@ fn test_version() {
     let versions = libcurl_version_info();
     eprintln!("{:?}", versions);
 }
+
+// This test function can be used to reproduce bug
+#[test]
+fn test_libcurl_directly() {
+    use curl;
+    use std::io::{stdout, Write};
+
+    let mut easy = curl::easy::Easy::new();
+    easy.url("http://localhost:8000/hello").unwrap();
+    easy.write_function(|data| {
+        stdout().write_all(data).unwrap();
+        Ok(data.len())
+    })
+    .unwrap();
+    easy.perform().unwrap();
+}
