@@ -1,6 +1,6 @@
 /*
- * hurl (https://hurl.dev)
- * Copyright (C) 2020 Orange
+ * Hurl (https://hurl.dev)
+ * Copyright (C) 2022 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -37,7 +37,7 @@ impl fmt::Display for Method {
 
 impl fmt::Display for Version {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "{}", self.value.to_string())
+        write!(f, "{}", self.value)
     }
 }
 
@@ -55,7 +55,7 @@ impl fmt::Display for VersionValue {
 
 impl fmt::Display for Status {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "{}", self.value.to_string())
+        write!(f, "{}", self.value)
     }
 }
 
@@ -63,7 +63,7 @@ impl fmt::Display for StatusValue {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             StatusValue::Any => write!(f, "*"),
-            StatusValue::Specific(v) => write!(f, "{}", v.to_string()),
+            StatusValue::Specific(v) => write!(f, "{}", v),
         }
     }
 }
@@ -82,7 +82,7 @@ impl fmt::Display for TemplateElement {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let s = match self {
             TemplateElement::String { value, .. } => value.clone(),
-            TemplateElement::Expression(value) => format!("{{{{{}}}}}", value.to_string()),
+            TemplateElement::Expression(value) => format!("{{{{{}}}}}", value),
         };
         write!(f, "{}", s)
     }
@@ -104,7 +104,7 @@ impl fmt::Display for CookiePath {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let mut buf = self.name.to_string();
         if let Some(attribute) = self.attribute.clone() {
-            let s = format!("[{}]", attribute.to_string());
+            let s = format!("[{}]", attribute);
             buf.push_str(s.as_str());
         }
         write!(f, "{}", buf)
@@ -137,6 +137,12 @@ impl fmt::Display for Hex {
     }
 }
 
+impl fmt::Display for Regex {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.inner)
+    }
+}
+
 impl PredicateFuncValue {
     pub fn name(&self) -> String {
         match self {
@@ -205,7 +211,7 @@ mod tests {
     fn whitespace() -> Whitespace {
         Whitespace {
             value: "".to_string(),
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         }
     }
 
@@ -214,7 +220,7 @@ mod tests {
             space0: whitespace(),
             variable: Variable {
                 name: "name".to_string(),
-                source_info: SourceInfo::init(0, 0, 0, 0),
+                source_info: SourceInfo::new(0, 0, 0, 0),
             },
             space1: whitespace(),
         }
@@ -234,7 +240,7 @@ mod tests {
                     encoded: "!".to_string(),
                 },
             ],
-            source_info: SourceInfo::init(0, 0, 0, 0),
+            source_info: SourceInfo::new(0, 0, 0, 0),
         }
     }
 
@@ -289,7 +295,7 @@ mod tests {
                         value: "LSID".to_string(),
                         encoded: "unused".to_string(),
                     }],
-                    source_info: SourceInfo::init(0, 0, 0, 0),
+                    source_info: SourceInfo::new(0, 0, 0, 0),
                 },
                 attribute: Some(CookieAttribute {
                     space0: whitespace(),
