@@ -1,6 +1,6 @@
 /*
- * hurl (https://hurl.dev)
- * Copyright (C) 2020 Orange
+ * Hurl (https://hurl.dev)
+ * Copyright (C) 2022 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -43,7 +43,7 @@ pub enum ParseError {
     Xml {},
     Predicate,
     PredicateValue,
-    RegexExpr,
+    RegexExpr { message: String },
 
     Unexpected { character: String },
     Eof {},
@@ -60,6 +60,7 @@ pub enum ParseError {
     InvalidCookieAttribute,
     OddNumberOfHexDigits,
     UrlIllegalCharacter(char),
+    InvalidOption,
 }
 
 impl Error {
