@@ -1,6 +1,6 @@
 /*
- * hurl (https://hurl.dev)
- * Copyright (C) 2020 Orange
+ * Hurl (https://hurl.dev)
+ * Copyright (C) 2022 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,11 +17,11 @@
  */
 use crate::ast::HurlFile;
 
-pub type ParseResult<'a, T> = std::result::Result<T, Error>;
-pub type ParseFunc<'a, T> = fn(&mut reader::Reader) -> ParseResult<'a, T>;
+pub type ParseResult<'a, T> = Result<T, Error>;
+pub type ParseFunc<'a, T> = fn(&mut Reader) -> ParseResult<'a, T>;
 
 pub fn parse_hurl_file(s: &str) -> ParseResult<'static, HurlFile> {
-    let mut reader = reader::Reader::init(s);
+    let mut reader = Reader::init(s);
     parsers::hurl_file(&mut reader)
 }
 
@@ -39,6 +39,7 @@ mod combinators;
 mod cookiepath;
 mod error;
 mod expr;
+mod filename;
 mod json;
 mod parsers;
 mod predicate;
