@@ -53,6 +53,7 @@ fn request_section(reader: &mut Reader) -> ParseResult<'static, Section> {
         "FormParams" => section_value_form_params(reader)?,
         "MultipartFormData" => section_value_multipart_form_data(reader)?,
         "Cookies" => section_value_cookies(reader)?,
+        "Options" => section_value_options(reader)?,
         _ => {
             return Err(Error {
                 pos: Pos {
@@ -150,6 +151,11 @@ fn section_value_asserts(reader: &mut Reader) -> ParseResult<'static, SectionVal
     Ok(SectionValue::Asserts(asserts))
 }
 
+fn section_value_options(reader: &mut Reader) -> ParseResult<'static, SectionValue> {
+    let options = zero_or_more(option, reader)?;
+    Ok(SectionValue::Options(options))
+}
+
 fn cookie(reader: &mut Reader) -> ParseResult<'static, Cookie> {
     // let start = reader.state.clone();
     let line_terminators = optional_line_terminators(reader)?;
@@ -158,7 +164,7 @@ fn cookie(reader: &mut Reader) -> ParseResult<'static, Cookie> {
     let space1 = zero_or_more_spaces(reader)?;
     recover(|p1| literal(":", p1), reader)?;
     let space2 = zero_or_more_spaces(reader)?;
-    let value = cookie_value(reader);
+    let value = unquoted_template(reader)?;
     let line_terminator0 = line_terminator(reader)?;
     Ok(Cookie {
         line_terminators,
@@ -171,23 +177,6 @@ fn cookie(reader: &mut Reader) -> ParseResult<'static, Cookie> {
     })
 }
 
-///
-/// The cookie-value must not be wrapped within double-quotes
-/// This is optional in the spec, if you really want to include these optional quotes, you must use directly the header Cookie
-///
-fn cookie_value(reader: &mut Reader) -> CookieValue {
-    //let start = reader.state.clone();
-    let value = reader.read_while(|c| {
-        c.is_ascii_alphanumeric()
-            || vec![
-                '!', '#', '$', '%', '&', '\'', '(', ')', '*', '+', '-', '.', '/', ':', '<', '=',
-                '>', '?', '@', '[', ']', '^', '_', '`', '~', '|', '"', ';', ',',
-            ]
-            .contains(c)
-    });
-    CookieValue { value }
-}
-
 fn multipart_param(reader: &mut Reader) -> ParseResult<'static, MultipartParam> {
     let save = reader.state.clone();
     match file_param(reader) {
@@ -352,6 +341,266 @@ fn assert(reader: &mut Reader) -> ParseResult<'static, Assert> {
     })
 }
 
+fn option(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    choice(
+        vec![
+            option_cacert,
+            option_compressed,
+            option_insecure,
+            option_follow_location,
+            option_max_redirect,
+            option_variable,
+            option_verbose,
+            option_very_verbose,
+        ],
+        reader,
+    )
+}
+
+fn option_cacert(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("cacert", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let f = filename::parse(reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+
+    let option = CaCertificateOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        filename: f,
+        line_terminator0,
+    };
+
+    Ok(EntryOption::CaCertificate(option))
+}
+
+fn option_compressed(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("compressed", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let value = nonrecover(boolean, reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+
+    let option = CompressedOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        value,
+        line_terminator0,
+    };
+
+    Ok(EntryOption::Compressed(option))
+}
+
+fn option_insecure(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("insecure", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let value = nonrecover(boolean, reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+
+    let option = InsecureOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        value,
+        line_terminator0,
+    };
+
+    Ok(EntryOption::Insecure(option))
+}
+
+fn option_follow_location(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("location", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let value = nonrecover(boolean, reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+
+    let option = FollowLocationOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        value,
+        line_terminator0,
+    };
+
+    Ok(EntryOption::FollowLocation(option))
+}
+
+fn option_max_redirect(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("max-redirs", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let value = nonrecover(natural, reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+
+    // FIXME: try to not unwrap redirect value
+    // and returns an error if not possible
+    let option = MaxRedirectOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        value: usize::try_from(value).unwrap(),
+        line_terminator0,
+    };
+
+    Ok(EntryOption::MaxRedirect(option))
+}
+
+fn option_variable(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("variable", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let value = variable_definition(reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+    let option = VariableOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        value,
+        line_terminator0,
+    };
+    Ok(EntryOption::Variable(option))
+}
+
+fn variable_definition(reader: &mut Reader) -> ParseResult<'static, VariableDefinition> {
+    let name = variable_name(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    literal("=", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    let value = variable_value(reader)?;
+    Ok(VariableDefinition {
+        name,
+        space0,
+        space1,
+        value,
+    })
+}
+
+fn variable_name(reader: &mut Reader) -> ParseResult<'static, String> {
+    let start = reader.state.clone();
+    let name = reader.read_while(|c| c.is_alphanumeric() || *c == '_' || *c == '-');
+    if name.is_empty() {
+        return Err(Error {
+            pos: start.pos,
+            recoverable: false,
+            inner: ParseError::Expecting {
+                value: "variable name".to_string(),
+            },
+        });
+    }
+    Ok(name)
+}
+
+fn variable_value(reader: &mut Reader) -> ParseResult<'static, VariableValue> {
+    choice(
+        vec![
+            |p1| match null(p1) {
+                Ok(()) => Ok(VariableValue::Null {}),
+                Err(e) => Err(e),
+            },
+            |p1| match boolean(p1) {
+                Ok(value) => Ok(VariableValue::Bool(value)),
+                Err(e) => Err(e),
+            },
+            |p1| match float(p1) {
+                Ok(value) => Ok(VariableValue::Float(value)),
+                Err(e) => Err(e),
+            },
+            |p1| match integer(p1) {
+                Ok(value) => Ok(VariableValue::Integer(value)),
+                Err(e) => Err(e),
+            },
+            |p1| match quoted_template(p1) {
+                Ok(value) => Ok(VariableValue::String(value)),
+                Err(e) => Err(e),
+            },
+            |p1| match unquoted_template(p1) {
+                Ok(value) => Ok(VariableValue::String(value)),
+                Err(e) => Err(e),
+            },
+        ],
+        reader,
+    )
+    .map_err(|e| Error {
+        pos: e.pos,
+        recoverable: false,
+        inner: ParseError::Expecting {
+            value: "variable value".to_string(),
+        },
+    })
+}
+
+fn option_verbose(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("verbose", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let value = nonrecover(boolean, reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+
+    let option = VerboseOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        value,
+        line_terminator0,
+    };
+
+    Ok(EntryOption::Verbose(option))
+}
+
+fn option_very_verbose(reader: &mut Reader) -> ParseResult<'static, EntryOption> {
+    let line_terminators = optional_line_terminators(reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
+    try_literal("very-verbose", reader)?;
+    let space1 = zero_or_more_spaces(reader)?;
+    try_literal(":", reader)?;
+    let space2 = zero_or_more_spaces(reader)?;
+    let value = nonrecover(boolean, reader)?;
+    let line_terminator0 = line_terminator(reader)?;
+
+    let option = VeryVerboseOption {
+        line_terminators,
+        space0,
+        space1,
+        space2,
+        value,
+        line_terminator0,
+    };
+
+    Ok(EntryOption::VeryVerbose(option))
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -374,31 +623,31 @@ mod tests {
                 line_terminators: vec![],
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 1, 1, 1),
+                    source_info: SourceInfo::new(1, 1, 1, 1),
                 },
                 line_terminator0: LineTerminator {
                     space0: Whitespace {
                         value: String::from(""),
-                        source_info: SourceInfo::init(1, 10, 1, 10),
+                        source_info: SourceInfo::new(1, 10, 1, 10),
                     },
                     comment: None,
                     newline: Whitespace {
                         value: String::from("\n"),
-                        source_info: SourceInfo::init(1, 10, 2, 1),
+                        source_info: SourceInfo::new(1, 10, 2, 1),
                     },
                 },
                 value: SectionValue::Asserts(vec![Assert {
                     line_terminators: vec![],
                     space0: Whitespace {
                         value: String::from(""),
-                        source_info: SourceInfo::init(2, 1, 2, 1),
+                        source_info: SourceInfo::new(2, 1, 2, 1),
                     },
                     query: Query {
-                        source_info: SourceInfo::init(2, 1, 2, 18),
+                        source_info: SourceInfo::new(2, 1, 2, 18),
                         value: QueryValue::Header {
                             space0: Whitespace {
                                 value: String::from(" "),
-                                source_info: SourceInfo::init(2, 7, 2, 8),
+                                source_info: SourceInfo::new(2, 7, 2, 8),
                             },
                             name: Template {
                                 quotes: true,
@@ -406,27 +655,27 @@ mod tests {
                                     value: "Location".to_string(),
                                     encoded: "Location".to_string(),
                                 }],
-                                source_info: SourceInfo::init(2, 8, 2, 18),
+                                source_info: SourceInfo::new(2, 8, 2, 18),
                             },
                         },
                         subquery: None,
                     },
                     space1: Whitespace {
                         value: String::from(" "),
-                        source_info: SourceInfo::init(2, 18, 2, 19),
+                        source_info: SourceInfo::new(2, 18, 2, 19),
                     },
                     predicate: Predicate {
                         not: false,
                         space0: Whitespace {
                             value: String::from(""),
-                            source_info: SourceInfo::init(2, 19, 2, 19),
+                            source_info: SourceInfo::new(2, 19, 2, 19),
                         },
                         predicate_func: PredicateFunc {
-                            source_info: SourceInfo::init(2, 19, 2, 45),
+                            source_info: SourceInfo::new(2, 19, 2, 45),
                             value: PredicateFuncValue::Equal {
                                 space0: Whitespace {
                                     value: String::from(" "),
-                                    source_info: SourceInfo::init(2, 25, 2, 26),
+                                    source_info: SourceInfo::new(2, 25, 2, 26),
                                 },
                                 value: PredicateValue::String(Template {
                                     quotes: true,
@@ -434,7 +683,7 @@ mod tests {
                                         value: "https://google.fr".to_string(),
                                         encoded: "https://google.fr".to_string(),
                                     }],
-                                    source_info: SourceInfo::init(2, 26, 2, 45),
+                                    source_info: SourceInfo::new(2, 26, 2, 45),
                                 }),
                                 operator: false,
                             },
@@ -443,16 +692,16 @@ mod tests {
                     line_terminator0: LineTerminator {
                         space0: Whitespace {
                             value: String::from(""),
-                            source_info: SourceInfo::init(2, 45, 2, 45),
+                            source_info: SourceInfo::new(2, 45, 2, 45),
                         },
                         comment: None,
                         newline: Whitespace {
                             value: String::from("\n"),
-                            source_info: SourceInfo::init(2, 45, 3, 1),
+                            source_info: SourceInfo::new(2, 45, 3, 1),
                         },
                     },
                 }]),
-                source_info: SourceInfo::init(1, 1, 1, 10),
+                source_info: SourceInfo::new(1, 1, 1, 10),
             }
         );
     }
@@ -488,38 +737,265 @@ mod tests {
         let mut reader = Reader::init("Foo: Bar");
         let c = cookie(&mut reader).unwrap();
         assert_eq!(c.name.value, String::from("Foo"));
-        assert_eq!(c.value.value, String::from("Bar"));
+        assert_eq!(
+            c.value,
+            Template {
+                quotes: false,
+                elements: vec![TemplateElement::String {
+                    value: "Bar".to_string(),
+                    encoded: "Bar".to_string(),
+                }],
+                source_info: SourceInfo::new(1, 6, 1, 9),
+            }
+        );
+    }
+
+    #[test]
+    fn test_option_insecure() {
+        let mut reader = Reader::init("insecure: true");
+        let option = option_insecure(&mut reader).unwrap();
+        assert_eq!(
+            option,
+            EntryOption::Insecure(InsecureOption {
+                line_terminators: vec![],
+                space0: Whitespace {
+                    value: "".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 1 },
+                        end: Pos { line: 1, column: 1 },
+                    },
+                },
+                space1: Whitespace {
+                    value: "".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 9 },
+                        end: Pos { line: 1, column: 9 },
+                    },
+                },
+                space2: Whitespace {
+                    value: " ".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos {
+                            line: 1,
+                            column: 10,
+                        },
+                        end: Pos {
+                            line: 1,
+                            column: 11,
+                        },
+                    },
+                },
+                value: true,
+                line_terminator0: LineTerminator {
+                    space0: Whitespace {
+                        value: "".to_string(),
+                        source_info: SourceInfo {
+                            start: Pos {
+                                line: 1,
+                                column: 15,
+                            },
+                            end: Pos {
+                                line: 1,
+                                column: 15,
+                            },
+                        },
+                    },
+                    comment: None,
+                    newline: Whitespace {
+                        value: "".to_string(),
+                        source_info: SourceInfo {
+                            start: Pos {
+                                line: 1,
+                                column: 15,
+                            },
+                            end: Pos {
+                                line: 1,
+                                column: 15,
+                            },
+                        },
+                    },
+                },
+            })
+        );
+    }
+
+    #[test]
+    fn test_option_insecure_error() {
+        let mut reader = Reader::init("insecure: error");
+        let error = option_insecure(&mut reader).err().unwrap();
+        assert_eq!(error.recoverable, false)
+    }
+
+    #[test]
+    fn test_option_cacert() {
+        let mut reader = Reader::init("cacert: /home/foo/cert.pem");
+        let option = option_cacert(&mut reader).unwrap();
+        assert_eq!(
+            option,
+            EntryOption::CaCertificate(CaCertificateOption {
+                line_terminators: vec![],
+                space0: Whitespace {
+                    value: "".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 1 },
+                        end: Pos { line: 1, column: 1 },
+                    },
+                },
+                space1: Whitespace {
+                    value: "".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 7 },
+                        end: Pos { line: 1, column: 7 },
+                    },
+                },
+                space2: Whitespace {
+                    value: " ".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 8 },
+                        end: Pos { line: 1, column: 9 },
+                    },
+                },
+                filename: Filename {
+                    value: "/home/foo/cert.pem".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 9 },
+                        end: Pos {
+                            line: 1,
+                            column: 27,
+                        },
+                    },
+                },
+                line_terminator0: LineTerminator {
+                    space0: Whitespace {
+                        value: "".to_string(),
+                        source_info: SourceInfo {
+                            start: Pos {
+                                line: 1,
+                                column: 27,
+                            },
+                            end: Pos {
+                                line: 1,
+                                column: 27,
+                            },
+                        },
+                    },
+                    comment: None,
+                    newline: Whitespace {
+                        value: "".to_string(),
+                        source_info: SourceInfo {
+                            start: Pos {
+                                line: 1,
+                                column: 27,
+                            },
+                            end: Pos {
+                                line: 1,
+                                column: 27,
+                            },
+                        },
+                    },
+                },
+            })
+        );
+    }
+
+    #[test]
+    fn test_option_cacert_error() {
+        let mut reader = Reader::init("cacert: ###");
+        let error = option_cacert(&mut reader).err().unwrap();
+        assert_eq!(error.recoverable, false)
+    }
+
+    #[test]
+    fn test_variable_definition() {
+        let mut reader = Reader::init("a=1");
+        assert_eq!(
+            variable_definition(&mut reader).unwrap(),
+            VariableDefinition {
+                name: "a".to_string(),
+                space0: Whitespace {
+                    value: "".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 2 },
+                        end: Pos { line: 1, column: 2 },
+                    },
+                },
+                space1: Whitespace {
+                    value: "".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos { line: 1, column: 3 },
+                        end: Pos { line: 1, column: 3 },
+                    },
+                },
+                value: VariableValue::Integer(1),
+            }
+        );
+    }
+
+    #[test]
+    fn test_variable_value() {
+        let mut reader = Reader::init("null");
+        assert_eq!(variable_value(&mut reader).unwrap(), VariableValue::Null {});
+
+        let mut reader = Reader::init("true");
+        assert_eq!(
+            variable_value(&mut reader).unwrap(),
+            VariableValue::Bool(true)
+        );
+
+        let mut reader = Reader::init("1");
+        assert_eq!(
+            variable_value(&mut reader).unwrap(),
+            VariableValue::Integer(1)
+        );
+
+        let mut reader = Reader::init("toto");
+        assert_eq!(
+            variable_value(&mut reader).unwrap(),
+            VariableValue::String(Template {
+                quotes: false,
+                elements: vec![TemplateElement::String {
+                    value: "toto".to_string(),
+                    encoded: "toto".to_string()
+                }],
+                source_info: SourceInfo {
+                    start: Pos { line: 1, column: 1 },
+                    end: Pos { line: 1, column: 5 },
+                }
+            })
+        );
+        let mut reader = Reader::init("\"123\"");
+        assert_eq!(
+            variable_value(&mut reader).unwrap(),
+            VariableValue::String(Template {
+                quotes: true,
+                elements: vec![TemplateElement::String {
+                    value: "123".to_string(),
+                    encoded: "123".to_string()
+                }],
+                source_info: SourceInfo {
+                    start: Pos { line: 1, column: 1 },
+                    end: Pos { line: 1, column: 6 },
+                }
+            })
+        );
     }
 
     #[test]
     fn test_cookie_error() {
-        let mut reader = Reader::init("Foo: {Bar}");
+        let mut reader = Reader::init("Foo: {{Bar");
         let error = cookie(&mut reader).err().unwrap();
-        assert_eq!(error.pos, Pos { line: 1, column: 6 });
+        assert_eq!(
+            error.pos,
+            Pos {
+                line: 1,
+                column: 11,
+            }
+        );
         assert!(!error.recoverable);
         assert_eq!(
             error.inner,
             ParseError::Expecting {
-                value: "line_terminator".to_string()
-            }
-        );
-    }
-
-    #[test]
-    fn test_cookie_value() {
-        let mut reader = Reader::init("Bar");
-        assert_eq!(
-            cookie_value(&mut reader),
-            CookieValue {
-                value: String::from("Bar")
-            }
-        );
-
-        let mut reader = Reader::init("\"Bar\"");
-        assert_eq!(
-            cookie_value(&mut reader),
-            CookieValue {
-                value: String::from("\"Bar\"")
+                value: "}}".to_string()
             }
         );
     }
@@ -532,19 +1008,19 @@ mod tests {
             FileValue {
                 space0: Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(1, 6, 1, 6),
+                    source_info: SourceInfo::new(1, 6, 1, 6),
                 },
                 filename: Filename {
                     value: "hello.txt".to_string(),
-                    source_info: SourceInfo::init(1, 6, 1, 15),
+                    source_info: SourceInfo::new(1, 6, 1, 15),
                 },
                 space1: Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(1, 15, 1, 15),
+                    source_info: SourceInfo::new(1, 15, 1, 15),
                 },
                 space2: Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(1, 16, 1, 16),
+                    source_info: SourceInfo::new(1, 16, 1, 16),
                 },
                 content_type: None,
             }
@@ -555,19 +1031,19 @@ mod tests {
             FileValue {
                 space0: Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(1, 6, 1, 6),
+                    source_info: SourceInfo::new(1, 6, 1, 6),
                 },
                 filename: Filename {
                     value: "hello.txt".to_string(),
-                    source_info: SourceInfo::init(1, 6, 1, 15),
+                    source_info: SourceInfo::new(1, 6, 1, 15),
                 },
                 space1: Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(1, 15, 1, 15),
+                    source_info: SourceInfo::new(1, 15, 1, 15),
                 },
                 space2: Whitespace {
                     value: " ".to_string(),
-                    source_info: SourceInfo::init(1, 16, 1, 17),
+                    source_info: SourceInfo::new(1, 16, 1, 17),
                 },
                 content_type: Some("text/html".to_string()),
             }
@@ -609,17 +1085,17 @@ mod tests {
                 quotes: false,
                 value: String::from("url"),
                 encoded: String::from("url"),
-                source_info: SourceInfo::init(1, 1, 1, 4),
+                source_info: SourceInfo::new(1, 1, 1, 4),
             }
         );
         assert_eq!(
             capture0.query,
             Query {
-                source_info: SourceInfo::init(1, 6, 1, 23),
+                source_info: SourceInfo::new(1, 6, 1, 23),
                 value: QueryValue::Header {
                     space0: Whitespace {
                         value: String::from(" "),
-                        source_info: SourceInfo::init(1, 12, 1, 13),
+                        source_info: SourceInfo::new(1, 12, 1, 13),
                     },
                     name: Template {
                         quotes: true,
@@ -627,7 +1103,7 @@ mod tests {
                             value: "Location".to_string(),
                             encoded: "Location".to_string(),
                         }],
-                        source_info: SourceInfo::init(1, 13, 1, 23),
+                        source_info: SourceInfo::new(1, 13, 1, 23),
                     },
                 },
                 subquery: None,
@@ -643,11 +1119,11 @@ mod tests {
         assert_eq!(
             capture0.query,
             Query {
-                source_info: SourceInfo::init(1, 8, 1, 25),
+                source_info: SourceInfo::new(1, 8, 1, 25),
                 value: QueryValue::Header {
                     space0: Whitespace {
                         value: String::from(" "),
-                        source_info: SourceInfo::init(1, 14, 1, 15),
+                        source_info: SourceInfo::new(1, 14, 1, 15),
                     },
                     name: Template {
                         quotes: true,
@@ -655,30 +1131,30 @@ mod tests {
                             value: "Location".to_string(),
                             encoded: "Location".to_string(),
                         }],
-                        source_info: SourceInfo::init(1, 15, 1, 25),
+                        source_info: SourceInfo::new(1, 15, 1, 25),
                     },
                 },
 
                 subquery: Some((
                     Whitespace {
                         value: " ".to_string(),
-                        source_info: SourceInfo::init(1, 25, 1, 26),
+                        source_info: SourceInfo::new(1, 25, 1, 26),
                     },
                     Subquery {
-                        source_info: SourceInfo::init(1, 26, 1, 44),
+                        source_info: SourceInfo::new(1, 26, 1, 44),
                         value: SubqueryValue::Regex {
                             space0: Whitespace {
                                 value: " ".to_string(),
-                                source_info: SourceInfo::init(1, 31, 1, 32),
+                                source_info: SourceInfo::new(1, 31, 1, 32),
                             },
-                            expr: Template {
+                            value: RegexValue::Template(Template {
                                 quotes: true,
                                 elements: vec![TemplateElement::String {
                                     value: "token=(.*)".to_string(),
                                     encoded: "token=(.*)".to_string(),
                                 }],
-                                source_info: SourceInfo::init(1, 32, 1, 44),
-                            },
+                                source_info: SourceInfo::new(1, 32, 1, 44),
+                            }),
                         },
                     }
                 )),
@@ -695,13 +1171,13 @@ mod tests {
             error.pos,
             Pos {
                 line: 1,
-                column: 32
+                column: 32,
             }
         );
         assert_eq!(
             error.inner,
             ParseError::Expecting {
-                value: "\"".to_string()
+                value: "\" or /".to_string()
             }
         );
         assert!(!error.recoverable);
@@ -732,11 +1208,11 @@ mod tests {
         assert_eq!(
             assert0.query,
             Query {
-                source_info: SourceInfo::init(1, 1, 1, 18),
+                source_info: SourceInfo::new(1, 1, 1, 18),
                 value: QueryValue::Header {
                     space0: Whitespace {
                         value: String::from(" "),
-                        source_info: SourceInfo::init(1, 7, 1, 8),
+                        source_info: SourceInfo::new(1, 7, 1, 8),
                     },
                     name: Template {
                         quotes: true,
@@ -744,7 +1220,7 @@ mod tests {
                             value: "Location".to_string(),
                             encoded: "Location".to_string(),
                         }],
-                        source_info: SourceInfo::init(1, 8, 1, 18),
+                        source_info: SourceInfo::new(1, 8, 1, 18),
                     },
                 },
                 subquery: None,
@@ -762,14 +1238,14 @@ mod tests {
                 not: false,
                 space0: Whitespace {
                     value: String::from(""),
-                    source_info: SourceInfo::init(1, 21, 1, 21),
+                    source_info: SourceInfo::new(1, 21, 1, 21),
                 },
                 predicate_func: PredicateFunc {
-                    source_info: SourceInfo::init(1, 21, 1, 29),
+                    source_info: SourceInfo::new(1, 21, 1, 29),
                     value: PredicateFuncValue::Equal {
                         space0: Whitespace {
                             value: String::from(" "),
-                            source_info: SourceInfo::init(1, 27, 1, 28),
+                            source_info: SourceInfo::new(1, 27, 1, 28),
                         },
                         value: PredicateValue::Integer(5),
                         operator: false,
