@@ -24,6 +24,8 @@ pub use self::logger::{
 
 mod fs;
 mod logger;
+mod options;
+pub use self::options::{get_string, get_strings, has_flag};
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct CliError {
